@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2023                                             *
+ *   Copyright (C) 2019 - 2024                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2012 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -40,6 +40,7 @@
 #include "kingdom.h"
 #include "localevent.h"
 #include "monster.h"
+#include "pal.h"
 #include "race.h"
 #include "tools.h"
 #include "translations.h"
@@ -89,7 +90,8 @@ namespace
             uint32_t redistributeCount = isSameTroopType ? 1 : troopFrom.GetCount() / 2;
 
             bool useFastSplit = !isSameTroopType;
-            const uint32_t slots = Dialog::ArmySplitTroop( ( freeSlots > overallCount ? overallCount : freeSlots ), maxCount, redistributeCount, useFastSplit );
+            const uint32_t slots
+                = Dialog::ArmySplitTroop( ( freeSlots > overallCount ? overallCount : freeSlots ), maxCount, redistributeCount, useFastSplit, troopFrom.GetName() );
 
             if ( slots < 2 || slots > 6 )
                 return;
@@ -172,13 +174,12 @@ namespace
     }
 }
 
-ArmyBar::ArmyBar( Army * ptr, bool mini, bool ro, bool change /* false */ )
+ArmyBar::ArmyBar( Army * ptr, const bool miniSprites, const bool readOnly, const bool isEditMode /* false */, const bool saveLastTroop /* true */ )
     : spcursor( fheroes2::AGG::GetICN( ICN::STRIP, 1 ) )
-    , _army( nullptr )
-    , use_mini_sprite( mini )
-    , read_only( ro )
-    , can_change( change )
-    , _troopWindowOffsetY( 0 )
+    , use_mini_sprite( miniSprites )
+    , read_only( readOnly )
+    , can_change( isEditMode )
+    , _saveLastTroop( saveLastTroop )
 {
     if ( use_mini_sprite )
         SetBackground( { 43, 43 }, fheroes2::GetColorId( 0, 45, 0 ) );
@@ -224,6 +225,7 @@ void ArmyBar::SetBackground( const fheroes2::Size & sz, const uint8_t fillColor
 
     setSingleItemSize( sz );
 
+    backsf._disableTransformLayer();
     backsf.resize( sz.width, sz.height );
     backsf.fill( fillColor );
 
@@ -236,10 +238,22 @@ void ArmyBar::SetBackground( const fheroes2::Size & sz, const uint8_t fillColor
 
 void ArmyBar::RedrawBackground( const fheroes2::Rect & pos, fheroes2::Image & dstsf )
 {
-    if ( use_mini_sprite )
-        fheroes2::Blit( backsf, dstsf, pos.x, pos.y );
-    else
-        fheroes2::Blit( fheroes2::AGG::GetICN( ICN::STRIP, 2 ), dstsf, pos.x, pos.y );
+    if ( use_mini_sprite ) {
+        fheroes2::Copy( backsf, 0, 0, dstsf, pos );
+    }
+    else {
+        if ( can_change && !_saveLastTroop && _army->GetOccupiedSlotCount() == 0 ) {
+            // If none of army's slot is set within the Editor, then a default army will be applied at the game start.
+            fheroes2::ApplyPalette( fheroes2::AGG::GetICN( ICN::STRIP, 2 ), 0, 0, dstsf, pos.x, pos.y, pos.width, pos.height,
+                                    PAL::GetPalette( PAL::PaletteType::DARKENING ) );
+
+            const fheroes2::Text text( _( "Default\ntroop" ), fheroes2::FontType::normalWhite() );
+            text.drawInRoi( pos.x, pos.y + pos.height / 2 - 17, pos.width, dstsf, pos );
+        }
+        else {
+            fheroes2::Copy( fheroes2::AGG::GetICN( ICN::STRIP, 2 ), 0, 0, dstsf, pos );
+        }
+    }
 }
 
 void ArmyBar::RedrawItem( ArmyTroop & troop, const fheroes2::Rect & pos, bool selected, fheroes2::Image & dstsf )
@@ -343,7 +357,7 @@ bool ArmyBar::ActionBarCursor( ArmyTroop & troop )
 
 bool ArmyBar::ActionBarCursor( ArmyTroop & destTroop, ArmyTroop & selectedTroop )
 {
-    bool save_last_troop = ( selectedTroop.GetArmy()->getTotalCount() <= 1 ) && selectedTroop.GetArmy()->SaveLastTroop();
+    const bool saveLastTroop = _saveLastTroop && ( selectedTroop.GetArmy()->getTotalCount() <= 1 ) && selectedTroop.GetArmy()->SaveLastTroop();
 
     if ( destTroop.isValid() ) {
         if ( destTroop.GetID() != selectedTroop.GetID() ) {
@@ -351,15 +365,17 @@ bool ArmyBar::ActionBarCursor( ArmyTroop & destTroop, ArmyTroop & selectedTroop
             StringReplaceWithLowercase( msg, "%{name}", destTroop.GetName() );
             StringReplaceWithLowercase( msg, "%{name2}", selectedTroop.GetName() );
         }
-        else if ( save_last_troop )
+        else if ( saveLastTroop ) {
             msg = _( "Cannot move last troop" );
+        }
         else {
             msg = _( "Combine %{name} armies" );
             StringReplaceWithLowercase( msg, "%{name}", destTroop.GetName() );
         }
     }
-    else if ( save_last_troop )
+    else if ( saveLastTroop ) {
         msg = _( "Cannot move last troop" );
+    }
     else {
         if ( selectedTroop.GetCount() == 1 ) {
             msg = _( "Move the %{name}" );
@@ -454,10 +470,21 @@ bool ArmyBar::ActionBarLeftMouseSingleClick( ArmyTroop & troop )
             const Monster mons = Dialog::selectMonster( cur );
 
             if ( mons.isValid() ) {
+                std::string str = _( "Set %{monster} Count" );
+                StringReplace( str, "%{monster}", mons.GetName() );
+
+                fheroes2::Sprite surface;
+
+                if ( mons.isValid() ) {
+                    surface = fheroes2::AGG::GetICN( ICN::STRIP, 12 );
+                    fheroes2::renderMonsterFrame( mons, surface, { 6, 6 } );
+                }
+
                 uint32_t count = 1;
 
-                if ( Dialog::SelectCount( _( "Set Count" ), 1, 500000, count ) )
+                if ( Dialog::SelectCount( str, 1, 500000, count, 1, surface ) ) {
                     troop.Set( mons, count );
+                }
             }
         }
 
@@ -491,7 +518,7 @@ bool ArmyBar::ActionBarLeftMouseSingleClick( ArmyTroop & destTroop, ArmyTroop &
         }
         // destination troop has units and both troops are the same creature type
         else {
-            if ( selectedTroop.GetArmy()->SaveLastTroop() ) { // this is their army's only troop
+            if ( _saveLastTroop && selectedTroop.GetArmy()->SaveLastTroop() ) { // this is their army's only troop
                 // move all but one units to destination
                 destTroop.SetCount( destTroop.GetCount() + selectedTroop.GetCount() - 1 );
                 // leave a single unit behind
@@ -507,7 +534,7 @@ bool ArmyBar::ActionBarLeftMouseSingleClick( ArmyTroop & destTroop, ArmyTroop &
     }
     else {
         // destination troop is empty, source army would be emptied by moving all
-        if ( selectedTroop.GetArmy()->SaveLastTroop() ) {
+        if ( _saveLastTroop && selectedTroop.GetArmy()->SaveLastTroop() ) {
             // move all but one units into the empty destination slot
             destTroop.Set( selectedTroop, selectedTroop.GetCount() - 1 );
             selectedTroop.SetCount( 1 );
@@ -535,7 +562,7 @@ bool ArmyBar::ActionBarLeftMouseDoubleClick( ArmyTroop & troop )
         int flags = Dialog::BUTTONS;
 
         if ( !read_only ) {
-            if ( !_army->SaveLastTroop() ) {
+            if ( !_saveLastTroop || !_army->SaveLastTroop() ) {
                 flags |= Dialog::DISMISS;
             }
 
@@ -636,7 +663,7 @@ bool ArmyBar::ActionBarRightMouseHold( ArmyTroop & troop )
     if ( troop.isValid() ) {
         ResetSelected();
 
-        if ( can_change && !_army->SaveLastTroop() ) {
+        if ( can_change && ( !_saveLastTroop || !_army->SaveLastTroop() ) ) {
             troop.Reset();
         }
         else {
