@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2023                                             *
+ *   Copyright (C) 2019 - 2024                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -30,6 +30,7 @@
 #include "agg_image.h"
 #include "army_troop.h"
 #include "audio_manager.h"
+#include "castle_building_info.h"
 #include "cursor.h"
 #include "dialog.h"
 #include "game_hotkeys.h"
@@ -207,80 +208,13 @@ Funds BuildingInfo::GetCost( uint32_t build, int race )
     return payment;
 }
 
-int GetIndexBuildingSprite( uint32_t build )
-{
-    switch ( build ) {
-    case DWELLING_MONSTER1:
-        return 19;
-    case DWELLING_MONSTER2:
-        return 20;
-    case DWELLING_MONSTER3:
-        return 21;
-    case DWELLING_MONSTER4:
-        return 22;
-    case DWELLING_MONSTER5:
-        return 23;
-    case DWELLING_MONSTER6:
-        return 24;
-    case DWELLING_UPGRADE2:
-        return 25;
-    case DWELLING_UPGRADE3:
-        return 26;
-    case DWELLING_UPGRADE4:
-        return 27;
-    case DWELLING_UPGRADE5:
-        return 28;
-    case DWELLING_UPGRADE6:
-        return 29;
-    case DWELLING_UPGRADE7:
-        return 30;
-    case BUILD_MAGEGUILD1:
-    case BUILD_MAGEGUILD2:
-    case BUILD_MAGEGUILD3:
-    case BUILD_MAGEGUILD4:
-    case BUILD_MAGEGUILD5:
-        return 0;
-    case BUILD_THIEVESGUILD:
-        return 1;
-    case BUILD_SHRINE:
-    case BUILD_TAVERN:
-        return 2;
-    case BUILD_SHIPYARD:
-        return 3;
-    case BUILD_WELL:
-        return 4;
-    case BUILD_CASTLE:
-        return 6;
-    case BUILD_STATUE:
-        return 7;
-    case BUILD_LEFTTURRET:
-        return 8;
-    case BUILD_RIGHTTURRET:
-        return 9;
-    case BUILD_MARKETPLACE:
-        return 10;
-    case BUILD_WEL2:
-        return 11;
-    case BUILD_MOAT:
-        return 12;
-    case BUILD_SPEC:
-        return 13;
-    case BUILD_CAPTAIN:
-        return 15;
-    default:
-        break;
-    }
-
-    return 0;
-}
-
 BuildingInfo::BuildingInfo( const Castle & c, const building_t b )
     : castle( c )
     , building( b )
     , area( 0, 0, 135, 70 )
     , bcond( ALLOW_BUILD )
 {
-    if ( IsDwelling() )
+    if ( isDwelling( building ) )
         building = castle.GetActualDwelling( b );
 
     building = castle.isBuild( b ) ? castle.GetUpgradeBuilding( b ) : b;
@@ -295,33 +229,8 @@ BuildingInfo::BuildingInfo( const Castle & c, const building_t b )
     // generate description
     if ( BUILD_DISABLE == bcond )
         description = GetConditionDescription();
-    else if ( IsDwelling() ) {
-        description = _( "The %{building} produces %{monster}." );
-        StringReplace( description, "%{building}", Castle::GetStringBuilding( building, castle.GetRace() ) );
-        StringReplaceWithLowercase( description, "%{monster}", Monster( castle.GetRace(), building ).GetMultiName() );
-    }
-    else
-        description = Castle::GetDescriptionBuilding( building, castle.GetRace() );
-
-    switch ( building ) {
-    case BUILD_WELL:
-        StringReplace( description, "%{count}", Castle::GetGrownWell() );
-        break;
-
-    case BUILD_WEL2:
-        StringReplace( description, "%{count}", Castle::GetGrownWel2() );
-        break;
-
-    case BUILD_CASTLE:
-    case BUILD_STATUE:
-    case BUILD_SPEC: {
-        const Funds profit = ProfitConditions::FromBuilding( building, castle.GetRace() );
-        StringReplace( description, "%{count}", profit.gold );
-        break;
-    }
-
-    default:
-        break;
+    else {
+        description = getBuildingDescription( castle.GetRace(), building );
     }
 
     // fix area for captain
@@ -338,7 +247,7 @@ void BuildingInfo::SetPos( int32_t x, int32_t y )
     area.y = y;
 }
 
-bool BuildingInfo::IsDwelling() const
+bool BuildingInfo::isDwelling( const uint32_t building )
 {
     switch ( building ) {
     case DWELLING_MONSTER1:
@@ -360,6 +269,45 @@ bool BuildingInfo::IsDwelling() const
     return false;
 }
 
+std::string BuildingInfo::getBuildingDescription( const int race, const uint32_t buildingId )
+{
+    std::string description;
+
+    if ( isDwelling( buildingId ) ) {
+        description = _( "The %{building} produces %{monster}." );
+        StringReplace( description, "%{building}", Castle::GetStringBuilding( buildingId, race ) );
+        if ( race == Race::RAND ) {
+            StringReplace( description, "%{monster}", Monster::getRandomRaceMonstersName( buildingId ) );
+        }
+        else {
+            StringReplaceWithLowercase( description, "%{monster}", Monster( race, buildingId ).GetMultiName() );
+        }
+    }
+    else {
+        description = fheroes2::getBuildingDescription( race, static_cast<building_t>( buildingId ) );
+
+        switch ( buildingId ) {
+        case BUILD_WELL:
+            StringReplace( description, "%{count}", Castle::GetGrownWell() );
+            break;
+        case BUILD_WEL2:
+            StringReplace( description, "%{count}", Castle::GetGrownWel2() );
+            break;
+        case BUILD_CASTLE:
+        case BUILD_STATUE:
+        case BUILD_SPEC: {
+            const Funds profit = ProfitConditions::FromBuilding( buildingId, race );
+            StringReplace( description, "%{count}", profit.gold );
+            break;
+        }
+        default:
+            break;
+        }
+    }
+
+    return description;
+}
+
 void BuildingInfo::RedrawCaptain() const
 {
     fheroes2::Display & display = fheroes2::Display::instance();
@@ -400,7 +348,7 @@ void BuildingInfo::Redraw() const
     }
 
     fheroes2::Display & display = fheroes2::Display::instance();
-    const int index = GetIndexBuildingSprite( building );
+    const int index = fheroes2::getIndexBuildingSprite( static_cast<building_t>( building ) );
 
     const fheroes2::Sprite & buildingFrame = fheroes2::AGG::GetICN( ICN::BLDGXTRA, 0 );
     fheroes2::Blit( buildingFrame, display, area.x, area.y );
@@ -498,20 +446,7 @@ bool BuildingInfo::DialogBuyBuilding( bool buttons ) const
     const fheroes2::Text descriptionText( std::move( extendedDescription ), fheroes2::FontType::normalWhite() );
 
     // prepare requirement build string
-    std::string requirement;
-    const uint32_t requirementBuildingIds = castle.GetBuildingRequirement( building );
-    const char sep = '\n';
-
-    for ( uint32_t itr = 0x00000001; itr; itr <<= 1 )
-        if ( requirementBuildingIds & itr ) {
-            requirement.append( Castle::GetStringBuilding( itr, castle.GetRace() ) );
-            requirement += sep;
-        }
-
-    // Remove the last separator.
-    if ( !requirement.empty() ) {
-        requirement.pop_back();
-    }
+    std::string requirement = fheroes2::getBuildingRequirementString( castle.GetRace(), static_cast<building_t>( building ) );
 
     const bool requirementsPresent = !requirement.empty();
 
@@ -553,7 +488,8 @@ bool BuildingInfo::DialogBuyBuilding( bool buttons ) const
     fheroes2::Display & display = fheroes2::Display::instance();
     fheroes2::Blit( buildingFrame, display, pos.x, pos.y );
 
-    const fheroes2::Sprite & buildingImage = fheroes2::AGG::GetICN( ICN::Get4Building( castle.GetRace() ), GetIndexBuildingSprite( building ) );
+    const fheroes2::Sprite & buildingImage
+        = fheroes2::AGG::GetICN( ICN::Get4Building( castle.GetRace() ), fheroes2::getIndexBuildingSprite( static_cast<building_t>( building ) ) );
     pos.x = dialogRoi.x + ( dialogRoi.width - buildingImage.width() ) / 2;
     pos.y += 1;
     fheroes2::Blit( buildingImage, display, pos.x, pos.y );
@@ -655,26 +591,28 @@ std::string BuildingInfo::GetConditionDescription() const
 
     case BUILD_DISABLE:
         if ( building == BUILD_SHIPYARD ) {
-            res = _( "Cannot build %{name} because the castle is too far away from an ocean." );
+            res = _( "Cannot build %{name}. The castle is too far away from an ocean." );
             StringReplace( res, "%{name}", Castle::GetStringBuilding( BUILD_SHIPYARD, castle.GetRace() ) );
         }
         else {
-            res = _( "disable build." );
+            // TODO: Add future disabled buildings here when it is made possible in the Editor.
+            assert( 0 );
+            res = _( "This building has been disabled." );
         }
         break;
 
     case LACK_RESOURCES:
-        res = _( "Cannot afford %{name}." );
+        res = _( "Cannot afford the %{name}." );
         StringReplace( res, "%{name}", GetName() );
         break;
 
     case ALREADY_BUILT:
-        res = _( "%{name} is already built." );
+        res = _( "The %{name} is already built." );
         StringReplace( res, "%{name}", GetName() );
         break;
 
     case REQUIRES_BUILD:
-        res = _( "Cannot build %{name}." );
+        res = _( "Cannot build the %{name}." );
         StringReplace( res, "%{name}", GetName() );
         break;
 
@@ -712,20 +650,17 @@ void BuildingInfo::SetStatusMessage( StatusBar & bar ) const
     }
 }
 
-DwellingItem::DwellingItem( const Castle & castle, uint32_t dw )
-{
-    type = castle.GetActualDwelling( dw );
-    mons = Monster( castle.GetRace(), type );
-}
-
 DwellingsBar::DwellingsBar( Castle & cstl, const fheroes2::Size & sz )
     : castle( cstl )
     , backsf( sz.width, sz.height )
 {
     backsf.reset();
 
-    for ( uint32_t dw = DWELLING_MONSTER1; dw <= DWELLING_MONSTER6; dw <<= 1 )
-        content.emplace_back( castle, dw );
+    content.reserve( CountBits( DWELLING_MONSTERS ) );
+
+    for ( uint32_t dw = DWELLING_MONSTER1; dw <= DWELLING_MONSTER6; dw <<= 1 ) {
+        content.emplace_back( dw );
+    }
 
     SetContent( content );
 
@@ -740,42 +675,50 @@ void DwellingsBar::RedrawBackground( const fheroes2::Rect & pos, fheroes2::Image
 
 void DwellingsBar::RedrawItem( DwellingItem & dwl, const fheroes2::Rect & pos, fheroes2::Image & dstsf )
 {
-    const fheroes2::Sprite & mons32 = fheroes2::AGG::GetICN( ICN::MONS32, dwl.mons.GetSpriteIndex() );
+    const uint32_t dwType = castle.GetActualDwelling( dwl.dwType );
+    const Monster mons{ castle.GetRace(), dwType };
+
+    const fheroes2::Sprite & mons32 = fheroes2::AGG::GetICN( ICN::MONS32, mons.GetSpriteIndex() );
     fheroes2::Blit( mons32, dstsf, pos.x + ( pos.width - mons32.width() ) / 2, pos.y + ( pos.height - 3 - mons32.height() ) );
 
-    if ( castle.isBuild( dwl.type ) ) {
-        // Available units for hire.
-        fheroes2::Text text( std::to_string( castle.getMonstersInDwelling( dwl.type ) ), fheroes2::FontType::smallWhite() );
-        text.draw( pos.x + pos.width - text.width() - 3, pos.y + pos.height - text.height() + 1, dstsf );
-
-        uint32_t grown = dwl.mons.GetGrown();
-        if ( castle.isBuild( BUILD_WELL ) )
-            grown += Castle::GetGrownWell();
-        if ( castle.isBuild( BUILD_WEL2 ) && DWELLING_MONSTER1 == dwl.type )
-            grown += Castle::GetGrownWel2();
-
-        // Dwelling's growth.
-        text.set( "+" + std::to_string( grown ), fheroes2::FontType::smallYellow() );
-        text.draw( pos.x + pos.width - text.width() - 3, pos.y + 4, dstsf );
-    }
-    else {
+    if ( !castle.isBuild( dwType ) ) {
         fheroes2::Blit( fheroes2::AGG::GetICN( ICN::CSLMARKER, 0 ), dstsf, pos.x + pos.width - 10, pos.y + 4 );
+
+        return;
     }
+
+    // Units available for hire.
+    fheroes2::Text text( std::to_string( castle.getMonstersInDwelling( dwType ) ), fheroes2::FontType::smallWhite() );
+    text.draw( pos.x + pos.width - text.width() - 3, pos.y + pos.height - text.height() + 1, dstsf );
+
+    uint32_t grown = mons.GetGrown();
+    if ( castle.isBuild( BUILD_WELL ) ) {
+        grown += Castle::GetGrownWell();
+    }
+    if ( castle.isBuild( BUILD_WEL2 ) && DWELLING_MONSTER1 == dwType ) {
+        grown += Castle::GetGrownWel2();
+    }
+
+    // Dwelling's growth.
+    text.set( "+" + std::to_string( grown ), fheroes2::FontType::smallYellow() );
+    text.draw( pos.x + pos.width - text.width() - 3, pos.y + 4, dstsf );
 }
 
 bool DwellingsBar::ActionBarLeftMouseSingleClick( DwellingItem & dwl )
 {
-    if ( castle.isBuild( dwl.type ) ) {
-        castle.RecruitMonster( Dialog::RecruitMonster( dwl.mons, castle.getMonstersInDwelling( dwl.type ), true, -60 ) );
+    const uint32_t dwType = castle.GetActualDwelling( dwl.dwType );
+
+    if ( castle.isBuild( dwType ) ) {
+        castle.RecruitMonster( Dialog::RecruitMonster( { castle.GetRace(), dwType }, castle.getMonstersInDwelling( dwType ), true, -60 ) );
     }
     else if ( !castle.isBuild( BUILD_CASTLE ) )
         fheroes2::showStandardTextMessage( "", GetBuildConditionDescription( NEED_CASTLE ), Dialog::OK );
     else {
-        const BuildingInfo dwelling( castle, static_cast<building_t>( dwl.type ) );
+        const BuildingInfo dwelling( castle, static_cast<building_t>( dwType ) );
 
         if ( dwelling.DialogBuyBuilding( true ) ) {
             AudioManager::PlaySound( M82::BUILDTWN );
-            castle.BuyBuilding( dwl.type );
+            castle.BuyBuilding( dwType );
         }
     }
 
@@ -784,7 +727,9 @@ bool DwellingsBar::ActionBarLeftMouseSingleClick( DwellingItem & dwl )
 
 bool DwellingsBar::ActionBarRightMouseHold( DwellingItem & dwl )
 {
-    Dialog::DwellingInfo( dwl.mons, castle.getMonstersInDwelling( dwl.type ) );
+    const uint32_t dwType = castle.GetActualDwelling( dwl.dwType );
+
+    Dialog::DwellingInfo( { castle.GetRace(), dwType }, castle.getMonstersInDwelling( dwType ) );
 
     return true;
 }
