@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2023                                             *
+ *   Copyright (C) 2019 - 2024                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -27,6 +27,7 @@
 #include <array>
 #include <cstddef>
 #include <cstdint>
+#include <functional>
 #include <map>
 #include <set>
 #include <string>
@@ -50,7 +51,12 @@ namespace fheroes2
     class Image;
 }
 
-class Funds;
+namespace Maps::Map_Format
+{
+    struct CastleMetadata;
+}
+
+struct Funds;
 class HeroBase;
 class Heroes;
 class StreamBase;
@@ -138,6 +144,8 @@ public:
 
     void LoadFromMP2( const std::vector<uint8_t> & data );
 
+    void loadFromResurrectionMap( const Maps::Map_Format::CastleMetadata & metadata );
+
     Captain & GetCaptain()
     {
         return captain;
@@ -179,7 +187,7 @@ public:
     }
 
     // This method must be called only at the time of map loading and only for castles with empty names.
-    void setName( const std::set<std::string> & usedNames );
+    void setName( const std::set<std::string, std::less<>> & usedNames );
 
     int GetControl() const override;
 
@@ -258,7 +266,6 @@ public:
     }
 
     bool BuyBuilding( uint32_t );
-    uint32_t GetBuildingRequirement( uint32_t ) const;
 
     int CheckBuyBuilding( const uint32_t build ) const;
     static int GetAllBuildingStatus( const Castle & );
@@ -272,11 +279,10 @@ public:
     std::string GetDescriptionBuilding( uint32_t ) const;
 
     static const char * GetStringBuilding( uint32_t, int race );
-    static const char * GetDescriptionBuilding( uint32_t, int race );
 
     static int GetICNBuilding( uint32_t, int race );
     static int GetICNBoat( int race );
-    uint32_t GetUpgradeBuilding( uint32_t ) const;
+    uint32_t GetUpgradeBuilding( const uint32_t buildingId ) const;
 
     static bool PredicateIsCastle( const Castle * );
     static bool PredicateIsTown( const Castle * );
@@ -323,6 +329,8 @@ private:
     void _wellRedrawBackground( fheroes2::Image & background ) const;
     void _wellRedrawMonsterAnimation( const fheroes2::Rect & roi, std::array<fheroes2::RandomMonsterAnimation, CASTLEMAXMONSTER> & monsterAnimInfo ) const;
 
+    void _setDefaultBuildings();
+
     // Recruit maximum monsters from the castle. Returns 'true' if the recruit was made.
     bool _recruitCastleMax( const Troops & currentCastleArmy );
     bool RecruitMonsterFromDwelling( uint32_t dw, uint32_t count, bool force = false );
