@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2023                                             *
+ *   Copyright (C) 2019 - 2024                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2010 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -77,7 +77,7 @@ namespace
         SELECTED_SCENARIO_DIFFICULTY_HEIGHT = 20,
         SELECTED_SCENARIO_DESCRIPTION_OFFSET_X = 42,
         SELECTED_SCENARIO_DESCRIPTION_OFFSET_Y = 318,
-        SELECTED_SCENARIO_DESCRIPTION_WIDTH = 292,
+        SELECTED_SCENARIO_DESCRIPTION_BOX_WIDTH = 292,
         SELECTED_SCENARIO_DESCRIPTION_HEIGHT = 90,
         SELECTED_SCENARIO_GENERAL_OFFSET_Y = 265,
         // COMMON
@@ -95,7 +95,7 @@ namespace
     void mapInfo( const Maps::FileInfo & info )
     {
         // On some OSes like Windows, the path may contain '\' symbols. This symbol doesn't exist in the resources.
-        // To avoid this we have to replace all '\' symbols by '/' symbols.
+        // To avoid this we have to replace all '\' symbols with '/' symbols.
         std::string fullPath = info.filename;
         StringReplace( fullPath, "\\", "/" );
 
@@ -170,7 +170,7 @@ namespace
             msg = _( "Find a specific artifact." );
             break;
         case Maps::FileInfo::VICTORY_DEFEAT_OTHER_SIDE:
-            msg = _( "Your side defeats the opposing side." );
+            msg = _( "Your side must defeat the opposing side." );
             break;
         case Maps::FileInfo::VICTORY_COLLECT_ENOUGH_GOLD:
             msg = _( "Accumulate a large amount of gold." );
@@ -204,10 +204,15 @@ namespace
         return startCoordX + centerTransform;
     }
 
-    fheroes2::ButtonBase * SwitchPressedMapSizeButtons( fheroes2::Button & newlyPressedButton )
+    void renderFileName( const Maps::FileInfo & info, bool selected, const int32_t posX, const int32_t posY, fheroes2::Display & display )
     {
-        newlyPressedButton.press();
-        return &newlyPressedButton;
+        fheroes2::Text text( System::GetBasename( info.filename ), selected ? fheroes2::FontType::normalYellow() : fheroes2::FontType::normalWhite() );
+        text.fitToOneRow( SCENARIO_LIST_MAP_NAME_WIDTH );
+
+        const int32_t xCoordinate = posX + SCENARIO_LIST_MAP_NAME_OFFSET_X;
+        const int32_t yCoordinate = posY + MAP_LIST_ROW_SPACING_Y - 1;
+
+        text.draw( xCoordinate, yCoordinate, display );
     }
 }
 
@@ -234,7 +239,12 @@ void ScenarioListBox::_renderScenarioListItem( const Maps::FileInfo & info, fher
     fheroes2::Blit( _getPlayersCountIcon( info.kingdomColors ), display, _offsetX + SCENARIO_LIST_COUNT_PLAYERS_OFFSET_X, dsty );
     _renderMapIcon( info.width, display, _offsetX + SCENARIO_LIST_MAP_SIZE_OFFSET_X, dsty );
     fheroes2::Blit( _getMapTypeIcon( info.version ), display, _offsetX + SCENARIO_LIST_MAP_TYPE_OFFSET_X, dsty );
-    _renderMapName( info, current, dsty, display );
+    if ( _isForEditor ) {
+        renderFileName( info, current, _offsetX, dsty, display );
+    }
+    else {
+        _renderMapName( info, current, dsty, display );
+    }
     fheroes2::Blit( _getWinConditionsIcon( info.victoryConditionType ), display, _offsetX + SCENARIO_LIST_VICTORY_CONDITION_OFFSET_X, dsty );
     fheroes2::Blit( _getLossConditionsIcon( info.lossConditionType ), display, _offsetX + SCENARIO_LIST_LOSS_CONDITION_OFFSET_X, dsty );
 }
@@ -264,8 +274,9 @@ void ScenarioListBox::_renderSelectedScenarioInfo( fheroes2::Display & display,
                          dst.y + SELECTED_SCENARIO_DIFFICULTY_OFFSET_Y, display );
 
     fheroes2::Text descriptionText( info.description, fheroes2::FontType::normalWhite() );
-    descriptionText.draw( dst.x + SELECTED_SCENARIO_DESCRIPTION_OFFSET_X, dst.y + SELECTED_SCENARIO_DESCRIPTION_OFFSET_Y + 5, SELECTED_SCENARIO_DESCRIPTION_WIDTH - 2,
-                          display );
+    descriptionText.setUniformVerticalAlignment( false );
+    descriptionText.draw( dst.x + SELECTED_SCENARIO_DESCRIPTION_OFFSET_X + 4, dst.y + SELECTED_SCENARIO_DESCRIPTION_OFFSET_Y + 3,
+                          SELECTED_SCENARIO_DESCRIPTION_BOX_WIDTH - 8, display );
 }
 
 void ScenarioListBox::_renderMapName( const Maps::FileInfo & info, bool selected, const int32_t & baseYOffset, fheroes2::Display & display ) const
@@ -372,12 +383,12 @@ const fheroes2::Sprite & ScenarioListBox::_getLossConditionsIcon( const uint8_t
     return fheroes2::AGG::GetICN( ICN::REQUESTS, iconIndex );
 }
 
-void ScenarioListBox::ActionListDoubleClick( Maps::FileInfo & )
+void ScenarioListBox::ActionListDoubleClick( Maps::FileInfo & /* unused */ )
 {
-    selectOk = true;
+    _isDoubleClicked = true;
 }
 
-const Maps::FileInfo * Dialog::SelectScenario( const MapsFileInfoList & allMaps )
+const Maps::FileInfo * Dialog::SelectScenario( const MapsFileInfoList & allMaps, const bool isForEditor )
 {
     if ( allMaps.empty() ) {
         return nullptr;
@@ -445,7 +456,7 @@ const Maps::FileInfo * Dialog::SelectScenario( const MapsFileInfoList & allMaps
     const fheroes2::Rect curDifficulty( rt.x + SELECTED_SCENARIO_DIFFICULTY_OFFSET_X, rt.y + SELECTED_SCENARIO_DIFFICULTY_OFFSET_Y, SELECTED_SCENARIO_DIFFICULTY_WIDTH,
                                         SELECTED_SCENARIO_DIFFICULTY_HEIGHT );
     const fheroes2::Rect curDescription( rt.x + SELECTED_SCENARIO_DESCRIPTION_OFFSET_X, rt.y + SELECTED_SCENARIO_DESCRIPTION_OFFSET_Y,
-                                         SELECTED_SCENARIO_DESCRIPTION_WIDTH, SELECTED_SCENARIO_DESCRIPTION_HEIGHT );
+                                         SELECTED_SCENARIO_DESCRIPTION_BOX_WIDTH, SELECTED_SCENARIO_DESCRIPTION_HEIGHT );
 
     fheroes2::Button buttonOk( rt.x + 140, rt.y + 410, ICN::BUTTON_SMALL_OKAY_GOOD, 0, 1 );
 
@@ -455,12 +466,22 @@ const Maps::FileInfo * Dialog::SelectScenario( const MapsFileInfoList & allMaps
     fheroes2::Button buttonSelectXLarge( rt.x + 222, rt.y + MAP_SIZE_BUTTON_OFFSET_Y, ICN::BUTTON_MAPSIZE_XLARGE, 0, 1 );
     fheroes2::Button buttonSelectAll( rt.x + 284, rt.y + MAP_SIZE_BUTTON_OFFSET_Y, ICN::BUTTON_MAPSIZE_ALL, 0, 1 );
 
+    const auto drawAllButtons = [&buttonOk, &buttonSelectSmall, &buttonSelectMedium, &buttonSelectLarge, &buttonSelectXLarge, &buttonSelectAll]() {
+        buttonOk.draw();
+        buttonSelectSmall.draw();
+        buttonSelectMedium.draw();
+        buttonSelectLarge.draw();
+        buttonSelectXLarge.draw();
+        buttonSelectAll.draw();
+    };
+
     ScenarioListBox listbox( rt.getPosition() );
     listbox.SetScrollButtonUp( ICN::REQUESTS, 5, 6, { rt.x + 327, rt.y + 55 } );
     listbox.SetScrollButtonDn( ICN::REQUESTS, 7, 8, { rt.x + 327, rt.y + 217 } );
     listbox.setScrollBarArea( { rt.x + 328, rt.y + 73, 12, 140 } );
     listbox.SetAreaMaxItems( 9 ); // This has impact on displaying selected scenario info
     listbox.SetAreaItems( { rt.x + 55, rt.y + 55, 270, 175 } );
+    listbox.setForEditorMode( isForEditor );
 
     fheroes2::ButtonBase * currentPressedButton = nullptr;
 
@@ -509,68 +530,21 @@ const Maps::FileInfo * Dialog::SelectScenario( const MapsFileInfoList & allMaps
         currentPressedButton->press();
     }
 
-    fheroes2::OptionButtonGroup buttonGroup;
-    buttonGroup.addButton( &buttonSelectSmall );
-    buttonGroup.addButton( &buttonSelectMedium );
-    buttonGroup.addButton( &buttonSelectLarge );
-    buttonGroup.addButton( &buttonSelectXLarge );
-    buttonGroup.addButton( &buttonSelectAll );
-
     listbox.RedrawBackground( rt.getPosition() );
     listbox.Redraw();
 
-    buttonOk.draw();
-    buttonSelectSmall.draw();
-    buttonSelectMedium.draw();
-    buttonSelectLarge.draw();
-    buttonSelectXLarge.draw();
-    buttonSelectAll.draw();
+    drawAllButtons();
 
     display.render();
 
     while ( le.HandleEvents() ) {
-        le.MousePressLeft( buttonOk.area() ) ? buttonOk.drawOnPress() : buttonOk.drawOnRelease();
-
-        if ( le.MousePressLeft( buttonSelectSmall.area() ) ) {
-            buttonSelectSmall.drawOnPress();
-        }
-        else if ( currentPressedButton != &buttonSelectSmall ) {
-            buttonSelectSmall.drawOnRelease();
-        }
-
-        if ( le.MousePressLeft( buttonSelectMedium.area() ) ) {
-            buttonSelectMedium.drawOnPress();
-        }
-        else if ( currentPressedButton != &buttonSelectMedium ) {
-            buttonSelectMedium.drawOnRelease();
-        }
-
-        if ( le.MousePressLeft( buttonSelectLarge.area() ) ) {
-            buttonSelectLarge.drawOnPress();
-        }
-        else if ( currentPressedButton != &buttonSelectLarge ) {
-            buttonSelectLarge.drawOnRelease();
-        }
-
-        if ( le.MousePressLeft( buttonSelectXLarge.area() ) ) {
-            buttonSelectXLarge.drawOnPress();
-        }
-        else if ( currentPressedButton != &buttonSelectXLarge ) {
-            buttonSelectXLarge.drawOnRelease();
-        }
-
-        if ( le.MousePressLeft( buttonSelectAll.area() ) ) {
-            buttonSelectAll.drawOnPress();
-        }
-        else if ( currentPressedButton != &buttonSelectAll ) {
-            buttonSelectAll.drawOnRelease();
-        }
+        buttonOk.drawOnState( le.MousePressLeft( buttonOk.area() ) );
 
         listbox.QueueEventProcessing();
 
         bool needRedraw = false;
 
-        if ( le.MouseClickLeft( buttonOk.area() ) || Game::HotKeyPressEvent( Game::HotKeyEvent::DEFAULT_OKAY ) || listbox.selectOk ) {
+        if ( le.MouseClickLeft( buttonOk.area() ) || Game::HotKeyPressEvent( Game::HotKeyEvent::DEFAULT_OKAY ) || listbox.isDoubleClicked() ) {
             MapsFileInfoList::const_iterator it = std::find( allMaps.begin(), allMaps.end(), listbox.GetCurrent() );
             return ( it != allMaps.end() ) ? &( *it ) : nullptr;
         }
@@ -581,123 +555,148 @@ const Maps::FileInfo * Dialog::SelectScenario( const MapsFileInfoList & allMaps
 
         if ( le.MouseClickLeft( buttonSelectSmall.area() ) || HotKeyPressEvent( Game::HotKeyEvent::MAIN_MENU_MAP_SIZE_SMALL ) ) {
             if ( small.empty() ) {
-                fheroes2::showStandardTextMessage( "", _( "No maps exist at that size" ), Dialog::OK );
-                currentPressedButton->drawOnPress();
+                fheroes2::showStandardTextMessage( "", _( "No maps exist at that size." ), Dialog::OK );
             }
             else {
                 currentMapFilter = Maps::SMALL;
-                listbox.SelectMapSize( small, Maps::SMALL );
-                currentPressedButton = SwitchPressedMapSizeButtons( buttonSelectSmall );
-            }
 
-            needRedraw = true;
+                listbox.SelectMapSize( small, Maps::SMALL );
+
+                currentPressedButton = &buttonSelectSmall;
+                needRedraw = true;
+            }
         }
         else if ( le.MouseClickLeft( buttonSelectMedium.area() ) || HotKeyPressEvent( Game::HotKeyEvent::MAIN_MENU_MAP_SIZE_MEDIUM ) ) {
             if ( medium.empty() ) {
-                fheroes2::showStandardTextMessage( "", _( "No maps exist at that size" ), Dialog::OK );
-                currentPressedButton->drawOnPress();
+                fheroes2::showStandardTextMessage( "", _( "No maps exist at that size." ), Dialog::OK );
             }
             else {
                 currentMapFilter = Maps::MEDIUM;
-                listbox.SelectMapSize( medium, Maps::MEDIUM );
-                currentPressedButton = SwitchPressedMapSizeButtons( buttonSelectMedium );
-            }
 
-            needRedraw = true;
+                listbox.SelectMapSize( medium, Maps::MEDIUM );
+
+                currentPressedButton = &buttonSelectMedium;
+                needRedraw = true;
+            }
         }
         else if ( le.MouseClickLeft( buttonSelectLarge.area() ) || HotKeyPressEvent( Game::HotKeyEvent::MAIN_MENU_MAP_SIZE_LARGE ) ) {
             if ( large.empty() ) {
-                fheroes2::showStandardTextMessage( "", _( "No maps exist at that size" ), Dialog::OK );
-                currentPressedButton->drawOnPress();
+                fheroes2::showStandardTextMessage( "", _( "No maps exist at that size." ), Dialog::OK );
             }
             else {
                 currentMapFilter = Maps::LARGE;
-                listbox.SelectMapSize( large, Maps::LARGE );
-                currentPressedButton = SwitchPressedMapSizeButtons( buttonSelectLarge );
-            }
 
-            needRedraw = true;
+                listbox.SelectMapSize( large, Maps::LARGE );
+
+                currentPressedButton = &buttonSelectLarge;
+                needRedraw = true;
+            }
         }
         else if ( le.MouseClickLeft( buttonSelectXLarge.area() ) || HotKeyPressEvent( Game::HotKeyEvent::MAIN_MENU_MAP_SIZE_EXTRA_LARGE ) ) {
             if ( xlarge.empty() ) {
-                fheroes2::showStandardTextMessage( "", _( "No maps exist at that size" ), Dialog::OK );
-                currentPressedButton->drawOnPress();
+                fheroes2::showStandardTextMessage( "", _( "No maps exist at that size." ), Dialog::OK );
             }
             else {
                 currentMapFilter = Maps::XLARGE;
-                listbox.SelectMapSize( xlarge, Maps::XLARGE );
-                currentPressedButton = SwitchPressedMapSizeButtons( buttonSelectXLarge );
-            }
 
-            needRedraw = true;
+                listbox.SelectMapSize( xlarge, Maps::XLARGE );
+
+                currentPressedButton = &buttonSelectXLarge;
+                needRedraw = true;
+            }
         }
         else if ( le.MouseClickLeft( buttonSelectAll.area() ) || HotKeyPressEvent( Game::HotKeyEvent::MAIN_MENU_MAP_SIZE_ALL ) ) {
             currentMapFilter = Maps::ZERO;
-            listbox.SelectMapSize( all, Maps::ZERO );
-            currentPressedButton = SwitchPressedMapSizeButtons( buttonSelectAll );
 
+            listbox.SelectMapSize( all, Maps::ZERO );
+
+            currentPressedButton = &buttonSelectAll;
             needRedraw = true;
         }
 
-        if ( le.MousePressRight( buttonSelectSmall.area() ) )
+        // The currentPressedButton must be set correctly before the following button redrawing code block, otherwise, the map size selection button that has just been
+        // clicked will be redrawn in the released state for a short time.
+        buttonSelectSmall.drawOnState( le.MousePressLeft( buttonSelectSmall.area() ) || currentPressedButton == &buttonSelectSmall );
+        buttonSelectMedium.drawOnState( le.MousePressLeft( buttonSelectMedium.area() ) || currentPressedButton == &buttonSelectMedium );
+        buttonSelectLarge.drawOnState( le.MousePressLeft( buttonSelectLarge.area() ) || currentPressedButton == &buttonSelectLarge );
+        buttonSelectXLarge.drawOnState( le.MousePressLeft( buttonSelectXLarge.area() ) || currentPressedButton == &buttonSelectXLarge );
+        buttonSelectAll.drawOnState( le.MousePressLeft( buttonSelectAll.area() ) || currentPressedButton == &buttonSelectAll );
+
+        if ( le.MousePressRight( buttonSelectSmall.area() ) ) {
             ShowToolTip( _( "Small Maps" ), _( "View only maps of size small (36 x 36)." ) );
-        else if ( le.MousePressRight( buttonSelectMedium.area() ) )
+        }
+        else if ( le.MousePressRight( buttonSelectMedium.area() ) ) {
             ShowToolTip( _( "Medium Maps" ), _( "View only maps of size medium (72 x 72)." ) );
-        else if ( le.MousePressRight( buttonSelectLarge.area() ) )
+        }
+        else if ( le.MousePressRight( buttonSelectLarge.area() ) ) {
             ShowToolTip( _( "Large Maps" ), _( "View only maps of size large (108 x 108)." ) );
-        else if ( le.MousePressRight( buttonSelectXLarge.area() ) )
+        }
+        else if ( le.MousePressRight( buttonSelectXLarge.area() ) ) {
             ShowToolTip( _( "Extra Large Maps" ), _( "View only maps of size extra large (144 x 144)." ) );
-        else if ( le.MousePressRight( buttonSelectAll.area() ) )
+        }
+        else if ( le.MousePressRight( buttonSelectAll.area() ) ) {
             ShowToolTip( _( "All Maps" ), _( "View all maps, regardless of size." ) );
-        else if ( le.MousePressRight( countPlayers ) || le.MousePressRight( curCountPlayer ) )
+        }
+        else if ( le.MousePressRight( countPlayers ) || le.MousePressRight( curCountPlayer ) ) {
             ShowToolTip( _( "Players Icon" ),
-                         _( "Indicates how many players total are in the scenario. Any positions not occupied by humans will be occupied by computer players." ) );
-        else if ( le.MousePressRight( sizeMaps ) || le.MousePressRight( curMapSize ) )
+                         _( "Indicates how many players total are in the scenario. Any positions not occupied by human players will be occupied by computer players." ) );
+        }
+        else if ( le.MousePressRight( sizeMaps ) || le.MousePressRight( curMapSize ) ) {
             ShowToolTip( _( "Size Icon" ), _( "Indicates whether the map\nis small (36 x 36), medium\n(72 x 72), large (108 x 108),\nor extra large (144 x 144)." ) );
-        else if ( le.MousePressRight( mapTypes ) || le.MousePressRight( curMapType ) )
+        }
+        else if ( le.MousePressRight( mapTypes ) || le.MousePressRight( curMapType ) ) {
             // TODO: update this tooltip once the Editor is out for public.
             ShowToolTip( _( "Map Type" ), _( "Indicates whether the map is made for \"The Succession Wars\" or \"The Price of Loyalty\" version of the game." ) );
+        }
         else if ( le.MousePressRight( mapNames ) ) {
             const Maps::FileInfo * item = listbox.GetFromPosition( le.GetMouseCursor() );
-            if ( item )
+            if ( item ) {
                 mapInfo( *item );
+            }
         }
-        else if ( le.MousePressRight( curMapName ) )
+        else if ( le.MousePressRight( curMapName ) ) {
             ShowToolTip( _( "Selected Name" ), _( "The name of the currently selected map." ) );
+        }
         else if ( le.MousePressRight( victoryConds ) ) {
             const Maps::FileInfo * item = listbox.GetFromPosition( le.GetMouseCursor() );
-            if ( item )
+            if ( item ) {
                 VictoryConditionInfo( *item );
+            }
         }
         else if ( le.MousePressRight( lossConds ) ) {
             const Maps::FileInfo * item = listbox.GetFromPosition( le.GetMouseCursor() );
-            if ( item )
+            if ( item ) {
                 LossConditionInfo( *item );
+            }
         }
-        else if ( le.MousePressRight( curVictoryCond ) )
+        else if ( le.MousePressRight( curVictoryCond ) ) {
             VictoryConditionInfo( listbox.GetCurrent() );
-        else if ( le.MousePressRight( curLossCond ) )
+        }
+        else if ( le.MousePressRight( curLossCond ) ) {
             LossConditionInfo( listbox.GetCurrent() );
-        else if ( le.MousePressRight( curDifficulty ) )
+        }
+        else if ( le.MousePressRight( curDifficulty ) ) {
             ShowToolTip(
                 _( "Selected Map Difficulty" ),
                 _( "The map difficulty of the currently selected map. The map difficulty is determined by the scenario designer. More difficult maps might include more or stronger enemies, fewer resources, or other special conditions making things tougher for the human player." ) );
-        else if ( le.MousePressRight( curDescription ) )
+        }
+        else if ( le.MousePressRight( curDescription ) ) {
             ShowToolTip( _( "Selected Description" ), _( "The description of the currently selected map." ) );
-        else if ( le.MousePressRight( buttonOk.area() ) )
+        }
+        else if ( le.MousePressRight( buttonOk.area() ) ) {
             ShowToolTip( _( "Okay" ), _( "Accept the choice made." ) );
+        }
 
         if ( !needRedraw && !listbox.IsNeedRedraw() ) {
             continue;
         }
 
         listbox.Redraw();
-        buttonOk.draw();
-        buttonSelectSmall.draw();
-        buttonSelectMedium.draw();
-        buttonSelectLarge.draw();
-        buttonSelectXLarge.draw();
-        buttonSelectAll.draw();
+
+        // The map list box redraws the entire window as a background (including all the buttons), so we have to redraw these buttons once again to correctly reflect
+        // their current state and not mess up with localized labels on these buttons.
+        drawAllButtons();
+
         display.render();
     }
 
