@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2023                                             *
+ *   Copyright (C) 2019 - 2024                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -39,7 +39,7 @@
 #include "dialog_game_settings.h"
 #include "dialog_language_selection.h"
 #include "dialog_resolution.h"
-#include "editor.h"
+#include "editor_mainmenu.h"
 #include "game.h"
 #include "game_delays.h"
 #include "game_hotkeys.h"
@@ -78,7 +78,8 @@ namespace
         LOADGAME_DEFAULT = 5,
         HIGHSCORES_DEFAULT = 9,
         CREDITS_DEFAULT = 13,
-        QUIT_DEFAULT = 17
+        QUIT_DEFAULT = 17,
+        EDITOR_DEFAULT = 20
     };
 
     void outputMainMenuInTextSupportMode()
@@ -91,12 +92,17 @@ namespace
         COUT( "Press " << Game::getHotKeyNameByEventId( Game::HotKeyEvent::MAIN_MENU_HIGHSCORES ) << " to show High Scores." )
         COUT( "Press " << Game::getHotKeyNameByEventId( Game::HotKeyEvent::MAIN_MENU_CREDITS ) << " to show Credits." )
         COUT( "Press " << Game::getHotKeyNameByEventId( Game::HotKeyEvent::MAIN_MENU_SETTINGS ) << " to open Game Settings." )
+
+        if ( Game::isPriceOfLoyaltyCampaignPresent() ) {
+            COUT( "Press " << Game::getHotKeyNameByEventId( Game::HotKeyEvent::EDITOR_MAIN_MENU ) << " to open Editor." )
+        }
+
         COUT( "Press " << Game::getHotKeyNameByEventId( Game::HotKeyEvent::MAIN_MENU_QUIT ) << " or " << Game::getHotKeyNameByEventId( Game::HotKeyEvent::DEFAULT_CANCEL )
                        << " to Quit the game." )
     }
 }
 
-void Game::mainGameLoop( bool isFirstGameRun )
+void Game::mainGameLoop( bool isFirstGameRun, bool isProbablyDemoVersion )
 {
     fheroes2::GameMode result = fheroes2::GameMode::MAIN_MENU;
 
@@ -112,7 +118,8 @@ void Game::mainGameLoop( bool isFirstGameRun )
             isFirstGameRun = false;
             break;
         case fheroes2::GameMode::NEW_GAME:
-            result = Game::NewGame();
+            result = Game::NewGame( isProbablyDemoVersion );
+            isProbablyDemoVersion = false;
             break;
         case fheroes2::GameMode::LOAD_GAME:
             result = Game::LoadGame();
@@ -184,7 +191,7 @@ void Game::mainGameLoop( bool isFirstGameRun )
             result = Editor::menuMain();
             break;
         case fheroes2::GameMode::EDITOR_NEW_MAP:
-            result = Editor::menuNewMap();
+            result = Editor::menuNewFromScratchMap();
             break;
         case fheroes2::GameMode::EDITOR_LOAD_MAP:
             result = Editor::menuLoadMap();
@@ -203,7 +210,7 @@ void Game::mainGameLoop( bool isFirstGameRun )
     fheroes2::fadeOutDisplay();
 }
 
-fheroes2::GameMode Game::MainMenu( bool isFirstGameRun )
+fheroes2::GameMode Game::MainMenu( const bool isFirstGameRun )
 {
     // Stop all sounds, but not the music
     AudioManager::stopSounds();
@@ -234,7 +241,7 @@ fheroes2::GameMode Game::MainMenu( bool isFirstGameRun )
         else {
             fheroes2::showStandardTextMessage(
                 _( "Greetings!" ),
-                _( "Welcome to Heroes of Might and Magic II powered by the fheroes2 engine!\nBefore starting the game please select a game resolution." ), Dialog::OK );
+                _( "Welcome to Heroes of Might and Magic II powered by the fheroes2 engine!\nBefore starting the game, please select a game resolution." ), Dialog::OK );
             const bool isResolutionChanged = Dialog::SelectResolution();
             if ( isResolutionChanged ) {
                 fheroes2::drawMainMenuScreen();
@@ -266,6 +273,7 @@ fheroes2::GameMode Game::MainMenu( bool isFirstGameRun )
     fheroes2::Button buttonLoadGame( 0, 0, ICN::BTNSHNGL, LOADGAME_DEFAULT, LOADGAME_DEFAULT + 2 );
     fheroes2::Button buttonHighScores( 0, 0, ICN::BTNSHNGL, HIGHSCORES_DEFAULT, HIGHSCORES_DEFAULT + 2 );
     fheroes2::Button buttonCredits( 0, 0, ICN::BTNSHNGL, CREDITS_DEFAULT, CREDITS_DEFAULT + 2 );
+    fheroes2::Button buttonEditor( 0, 0, ICN::BTNSHNGL, EDITOR_DEFAULT, EDITOR_DEFAULT + 2 );
     fheroes2::Button buttonQuit( 0, 0, ICN::BTNSHNGL, QUIT_DEFAULT, QUIT_DEFAULT + 2 );
 
     const fheroes2::Sprite & lantern10 = fheroes2::AGG::GetICN( ICN::SHNGANIM, 0 );
@@ -294,11 +302,15 @@ fheroes2::GameMode Game::MainMenu( bool isFirstGameRun )
 
     uint32_t lantern_frame = 0;
 
-    std::vector<ButtonInfo> buttons{ { NEWGAME_DEFAULT, buttonNewGame, false, false },
-                                     { LOADGAME_DEFAULT, buttonLoadGame, false, false },
-                                     { HIGHSCORES_DEFAULT, buttonHighScores, false, false },
-                                     { CREDITS_DEFAULT, buttonCredits, false, false },
-                                     { QUIT_DEFAULT, buttonQuit, false, false } };
+    const bool isPOLPresent = Game::isPriceOfLoyaltyCampaignPresent();
+
+    std::vector<ButtonInfo> buttons{ ButtonInfo{ NEWGAME_DEFAULT, buttonNewGame, false, false }, ButtonInfo{ LOADGAME_DEFAULT, buttonLoadGame, false, false },
+                                     ButtonInfo{ HIGHSCORES_DEFAULT, buttonHighScores, false, false }, ButtonInfo{ CREDITS_DEFAULT, buttonCredits, false, false },
+                                     ButtonInfo{ QUIT_DEFAULT, buttonQuit, false, false } };
+
+    if ( isPOLPresent ) {
+        buttons.emplace_back( ButtonInfo{ EDITOR_DEFAULT, buttonEditor, false, false } );
+    }
 
     for ( size_t i = 0; le.MouseMotion() && i < buttons.size(); ++i ) {
         const fheroes2::Sprite & sprite = fheroes2::AGG::GetICN( ICN::BTNSHNGL, buttons[i].frame );
@@ -376,12 +388,8 @@ fheroes2::GameMode Game::MainMenu( bool isFirstGameRun )
 
             return fheroes2::GameMode::MAIN_MENU;
         }
-        else if ( conf.isEditorEnabled() && HotKeyPressEvent( HotKeyEvent::EDITOR_MAIN_MENU ) ) {
-            if ( Game::isPriceOfLoyaltyCampaignPresent() ) {
-                return fheroes2::GameMode::EDITOR_MAIN_MENU;
-            }
-
-            fheroes2::showStandardTextMessage( _( "Editor" ), _( "The Editor requires \"The Price of Loyalty\" expansion files to work." ), Dialog::OK );
+        else if ( isPOLPresent && ( HotKeyPressEvent( HotKeyEvent::EDITOR_MAIN_MENU ) || le.MouseClickLeft( buttonEditor.area() ) ) ) {
+            return fheroes2::GameMode::EDITOR_MAIN_MENU;
         }
 
         // right info
@@ -395,6 +403,9 @@ fheroes2::GameMode Game::MainMenu( bool isFirstGameRun )
             fheroes2::showStandardTextMessage( _( "High Scores" ), _( "View the high scores screen." ), Dialog::ZERO );
         else if ( le.MousePressRight( buttonNewGame.area() ) )
             fheroes2::showStandardTextMessage( _( "New Game" ), _( "Start a single or multi-player game." ), Dialog::ZERO );
+        else if ( isPOLPresent && le.MousePressRight( buttonEditor.area() ) ) {
+            fheroes2::showStandardTextMessage( _( "Editor" ), _( "Create new or modify existing maps." ), Dialog::ZERO );
+        }
         else if ( le.MousePressRight( settingsArea ) )
             fheroes2::showStandardTextMessage( _( "Game Settings" ), _( "Change language, resolution and settings of the game." ), Dialog::ZERO );
 
