@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2023                                             *
+ *   Copyright (C) 2019 - 2024                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2012 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -128,6 +128,17 @@ void Interface::AdventureMap::_startHeroMove( Heroes & hero )
     Game::passAnimationDelay( Game::DelayType::CURRENT_HERO_DELAY );
 }
 
+void Interface::AdventureMap::EventSwitchFocusedHero( const int32_t tileIndex )
+{
+    Heroes * selectedHero = world.GetTiles( tileIndex ).getHero();
+    const Heroes * currentHero = GetFocusHeroes();
+    if ( selectedHero == nullptr || selectedHero == currentHero || selectedHero->GetColor() != currentHero->GetColor() ) {
+        return;
+    }
+    SetFocus( selectedHero, false );
+    RedrawFocus();
+}
+
 void Interface::AdventureMap::EventNextHero()
 {
     const Kingdom & myKingdom = world.GetKingdom( Settings::Get().CurrentColor() );
@@ -174,7 +185,7 @@ fheroes2::GameMode Interface::AdventureMap::EventHeroMovement()
         if ( hero->GetPath().isValidForMovement() && hero->MayStillMove( false, true ) ) {
             _startHeroMove( *hero );
         }
-        else if ( MP2::isActionObject( hero->getObjectTypeUnderHero(), hero->isShipMaster() ) ) {
+        else if ( MP2::isInGameActionObject( hero->getObjectTypeUnderHero(), hero->isShipMaster() ) ) {
             return EventDefaultAction();
         }
     }
@@ -475,7 +486,7 @@ fheroes2::GameMode Interface::AdventureMap::EventDefaultAction()
     Heroes * hero = GetFocusHeroes();
 
     if ( hero ) {
-        if ( MP2::isActionObject( hero->getObjectTypeUnderHero(), hero->isShipMaster() ) ) {
+        if ( MP2::isInGameActionObject( hero->getObjectTypeUnderHero(), hero->isShipMaster() ) ) {
             hero->Action( hero->GetIndex() );
 
             // The action object can alter the status of the hero (e.g. Stables or Well) or
