@@ -25,6 +25,7 @@
 
 #include <algorithm>
 #include <cassert>
+#include <cmath>
 #include <cstdlib>
 #include <deque>
 #include <list>
@@ -321,6 +322,8 @@ Interface::GameArea::GameArea( BaseInterface & interface )
     , _mouseDraggingInitiated( false )
     , _mouseDraggingMovement( false )
     , _needRedrawByMouseDragging( false )
+    , _isFastScrollEnabled( false )
+    , _resetMousePositionForFastScroll( false )
 {
     // Do nothing.
 }
@@ -482,6 +485,8 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
     const int32_t roiToRenderMaxX = std::min( maxX + 2, world.w() );
     const int32_t roiToRenderMaxY = std::min( maxY + 2, world.h() );
 
+    const bool isEditor = _interface.isEditor();
+
     for ( int32_t posY = roiToRenderMinY; posY < roiToRenderMaxY; ++posY ) {
         for ( int32_t posX = roiToRenderMinX; posX < roiToRenderMaxX; ++posX ) {
             const Maps::Tiles & tile = world.GetTiles( posX, posY );
@@ -493,7 +498,7 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
 
             switch ( objectType ) {
             case MP2::OBJ_HERO: {
-                if ( _interface.isEditor() ) {
+                if ( isEditor ) {
                     const uint8_t alphaValue = getObjectAlphaValue( tile.GetIndex(), MP2::OBJ_HERO );
 
                     auto spriteInfo = getEditorHeroSpritesPerTile( tile );
@@ -540,8 +545,8 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
 
                 const uint8_t alphaValue = getObjectAlphaValue( tile.GetIndex(), MP2::OBJ_MONSTER );
 
-                auto spriteInfo = getMonsterSpritesPerTile( tile, _interface.isEditor() );
-                auto spriteShadowInfo = getMonsterShadowSpritesPerTile( tile, _interface.isEditor() );
+                auto spriteInfo = getMonsterSpritesPerTile( tile, isEditor );
+                auto spriteShadowInfo = getMonsterShadowSpritesPerTile( tile, isEditor );
 
                 populateStaticTileUnfitObjectInfo( tileUnfit, spriteInfo, spriteShadowInfo, tile.GetCenter(), alphaValue );
 
@@ -734,52 +739,50 @@ void Interface::GameArea::Redraw( fheroes2::Image & dst, int flag, bool isPuzzle
         }
     }
 
-    const bool drawTowns = ( flag & LEVEL_TOWNS );
+    bool drawPassabilities = ( flag & LEVEL_PASSABILITIES );
 
 #ifdef WITH_DEBUG
-    if ( IS_DEVEL() ) {
-        // redraw grid
-        if ( flag & LEVEL_ALL ) {
-            const int32_t friendColors = Players::FriendColors();
+    if ( IS_DEVEL() && ( flag & LEVEL_ALL ) ) {
+        drawPassabilities = true;
+    }
+#endif
 
-            for ( int32_t y = minY; y < maxY; ++y ) {
-                for ( int32_t x = minX; x < maxX; ++x ) {
-                    redrawPassable( world.GetTiles( x, y ), dst, friendColors, *this );
-                }
+    if ( drawPassabilities ) {
+        const int32_t friendColors = Players::FriendColors();
+
+        for ( int32_t y = minY; y < maxY; ++y ) {
+            for ( int32_t x = minX; x < maxX; ++x ) {
+                redrawPassable( world.GetTiles( x, y ), dst, friendColors, *this, isEditor );
             }
         }
     }
-    else {
-#endif
-        // redraw fog
-        if ( renderFog ) {
-            for ( int32_t y = minY; y < maxY; ++y ) {
-                for ( int32_t x = minX; x < maxX; ++x ) {
-                    const Maps::Tiles & tile = world.GetTiles( x, y );
+    else if ( renderFog ) {
+        const bool drawTowns = ( flag & LEVEL_TOWNS );
 
-                    if ( tile.getFogDirection() != Direction::UNKNOWN ) {
-                        drawFog( tile, dst, *this );
+        for ( int32_t y = minY; y < maxY; ++y ) {
+            for ( int32_t x = minX; x < maxX; ++x ) {
+                const Maps::Tiles & tile = world.GetTiles( x, y );
 
-                        if ( drawTowns ) {
-                            drawByObjectIcnType( tile, dst, *this, MP2::OBJ_ICN_TYPE_OBJNTWBA );
+                if ( tile.getFogDirection() != Direction::UNKNOWN ) {
+                    drawFog( tile, dst, *this );
 
-                            const MP2::MapObjectType objectType = tile.GetObject( false );
-                            if ( objectType == MP2::OBJ_CASTLE || objectType == MP2::OBJ_NON_ACTION_CASTLE ) {
-                                drawByObjectIcnType( tile, dst, *this, MP2::OBJ_ICN_TYPE_OBJNTOWN );
-                            }
+                    if ( drawTowns ) {
+                        drawByObjectIcnType( tile, dst, *this, MP2::OBJ_ICN_TYPE_OBJNTWBA );
+
+                        const MP2::MapObjectType objectType = tile.GetObject( false );
+                        if ( objectType == MP2::OBJ_CASTLE || objectType == MP2::OBJ_NON_ACTION_CASTLE ) {
+                            drawByObjectIcnType( tile, dst, *this, MP2::OBJ_ICN_TYPE_OBJNTOWN );
                         }
                     }
                 }
             }
         }
-#ifdef WITH_DEBUG
     }
-#endif
 
     updateObjectAnimationInfo();
 }
 
-void Interface::GameArea::renderTileAreaSelect( fheroes2::Image & dst, const int32_t startTile, const int32_t endTile ) const
+void Interface::GameArea::renderTileAreaSelect( fheroes2::Image & dst, const int32_t startTile, const int32_t endTile, const bool isActionObject ) const
 {
     if ( startTile < 0 || endTile < 0 ) {
         return;
@@ -799,10 +802,12 @@ void Interface::GameArea::renderTileAreaSelect( fheroes2::Image & dst, const int
     const int32_t limitedLineWidth = std::min( 2, overlappedRoi.width );
     const int32_t limitedLineHeight = std::min( 2, overlappedRoi.height );
 
-    fheroes2::Fill( dst, overlappedRoi.x, overlappedRoi.y, overlappedRoi.width, limitedLineHeight, 181 );
-    fheroes2::Fill( dst, overlappedRoi.x, overlappedRoi.y + 2, limitedLineWidth, overlappedRoi.height - 4, 181 );
-    fheroes2::Fill( dst, overlappedRoi.x, overlappedRoi.y + overlappedRoi.height - limitedLineHeight, overlappedRoi.width, limitedLineHeight, 181 );
-    fheroes2::Fill( dst, overlappedRoi.x + overlappedRoi.width - limitedLineWidth, overlappedRoi.y + 2, limitedLineWidth, overlappedRoi.height - 4, 181 );
+    const uint8_t color = ( isActionObject ? 115 : 181 );
+
+    fheroes2::Fill( dst, overlappedRoi.x, overlappedRoi.y, overlappedRoi.width, limitedLineHeight, color );
+    fheroes2::Fill( dst, overlappedRoi.x, overlappedRoi.y + 2, limitedLineWidth, overlappedRoi.height - 4, color );
+    fheroes2::Fill( dst, overlappedRoi.x, overlappedRoi.y + overlappedRoi.height - limitedLineHeight, overlappedRoi.width, limitedLineHeight, color );
+    fheroes2::Fill( dst, overlappedRoi.x + overlappedRoi.width - limitedLineWidth, overlappedRoi.y + 2, limitedLineWidth, overlappedRoi.height - 4, color );
 }
 
 void Interface::GameArea::updateMapFogDirections()
@@ -936,6 +941,75 @@ void Interface::GameArea::SetScroll( int direct )
     scrollTime.reset();
 }
 
+void Interface::GameArea::setFastScrollStatus( const bool enable )
+{
+    _isFastScrollEnabled = enable;
+    _resetMousePositionForFastScroll = true;
+}
+
+bool Interface::GameArea::mouseIndicatesFastScroll( const fheroes2::Point & mousePosition )
+{
+    const fheroes2::Display & display = fheroes2::Display::instance();
+    constexpr int32_t deadZone = 3;
+
+    // Remember the initial reference point for re-enabling checks later on.
+    if ( _resetMousePositionForFastScroll ) {
+        _mousePositionForFastScroll = mousePosition;
+        _resetMousePositionForFastScroll = false;
+    }
+
+    if ( Interface::BaseInterface::isScrollLeft( _mousePositionForFastScroll ) ) {
+        if ( mousePosition.x > _mousePositionForFastScroll.x ) {
+            // Movement is away from the border, we need to update the checking point.
+            _mousePositionForFastScroll = mousePosition;
+        }
+        else if ( mousePosition.x < _mousePositionForFastScroll.x || ( abs( mousePosition.y - _mousePositionForFastScroll.y ) > deadZone && mousePosition.x <= 0 ) ) {
+            // Movement is towards or along the border, we re-enable the fast scroll.
+            return true;
+        }
+    }
+    else if ( Interface::BaseInterface::isScrollRight( _mousePositionForFastScroll ) ) {
+        if ( mousePosition.x < _mousePositionForFastScroll.x ) {
+            // Movement is away from the border, we need to update the checking point.
+            _mousePositionForFastScroll = mousePosition;
+        }
+        else if ( mousePosition.x > _mousePositionForFastScroll.x
+                  || ( abs( mousePosition.y - _mousePositionForFastScroll.y ) > deadZone && mousePosition.x >= display.width() - 1 ) ) {
+            // Movement is towards or along the border, we re-enable the fast scroll.
+            return true;
+        }
+    }
+    else if ( Interface::BaseInterface::isScrollTop( _mousePositionForFastScroll ) ) {
+        if ( mousePosition.y > _mousePositionForFastScroll.y ) {
+            // Movement is away from the border, we need to update the checking point.
+            _mousePositionForFastScroll = mousePosition;
+        }
+        else if ( mousePosition.y < _mousePositionForFastScroll.y || ( abs( mousePosition.x - _mousePositionForFastScroll.x ) > deadZone && mousePosition.y <= 0 ) ) {
+            // Movement is towards or along the border, we re-enable the fast scroll.
+            return true;
+        }
+    }
+    else if ( Interface::BaseInterface::isScrollBottom( _mousePositionForFastScroll ) ) {
+        if ( mousePosition.y < _mousePositionForFastScroll.y ) {
+            // Movement is away from the border, we need to update the checking point.
+            _mousePositionForFastScroll = mousePosition;
+        }
+        else if ( mousePosition.y > _mousePositionForFastScroll.y
+                  || ( abs( mousePosition.x - _mousePositionForFastScroll.x ) > deadZone && mousePosition.y >= display.height() - 1 ) ) {
+            // Movement is towards or along the border, we re-enable the fast scroll.
+            return true;
+        }
+    }
+    else {
+        // We have left the scroll borders, fast scrolling can definitely be re-enabled.
+        return true;
+    }
+
+    // We haven't left the borders, but the direction of the mouse movement within the borders
+    // does not indicate that the user wants to perform the fast scroll right now.
+    return false;
+}
+
 void Interface::GameArea::QueueEventProcessing( bool isCursorOverGamearea )
 {
     LocalEvent & le = LocalEvent::Get();
@@ -956,7 +1030,7 @@ void Interface::GameArea::QueueEventProcessing( bool isCursorOverGamearea )
         _mouseDraggingMovement = true;
     }
 
-    if ( _mouseDraggingMovement ) {
+    if ( _mouseDraggingMovement && le.MousePressLeft( GetROI() ) ) {
         if ( _lastMouseDragPosition == mousePosition ) {
             _needRedrawByMouseDragging = false;
         }
@@ -1004,6 +1078,9 @@ void Interface::GameArea::QueueEventProcessing( bool isCursorOverGamearea )
     else if ( le.MousePressRight( tileROI ) ) {
         _interface.mouseCursorAreaPressRight( index );
     }
+    else if ( le.MouseLongPressLeft( tileROI ) ) {
+        _interface.mouseCursorAreaLongPressLeft( index );
+    }
 
     // The cursor may have moved after mouse click events.
     index = GetValidTileIdFromPoint( le.GetMouseCursor() );
