@@ -25,8 +25,6 @@
 
 #include <algorithm>
 #include <cassert>
-#include <cstddef>
-#include <memory>
 #include <ostream>
 #include <string>
 #include <vector>
@@ -105,7 +103,6 @@ bool HeroesStrongestArmy( const Heroes * h1, const Heroes * h2 )
 
 Kingdom::Kingdom()
     : color( Color::NONE )
-    , _lastBattleWinHeroID( 0 )
     , lost_town_days( 0 )
     , visited_tents_colors( 0 )
     , _topCastleInKingdomView( -1 )
@@ -114,26 +111,25 @@ Kingdom::Kingdom()
     // Do nothing.
 }
 
-void Kingdom::Init( int clr )
+void Kingdom::Init( const int clr )
 {
     clear();
+
     color = clr;
 
-    if ( Color::ALL & color ) {
-        // Difficulty calculation is different for campaigns. Difficulty affects only on starting resources for human players.
-        const Settings & configuration = Settings::Get();
-        const int difficultyLevel = ( configuration.isCampaignGameType() ? configuration.getCurrentMapInfo().difficulty : configuration.GameDifficulty() );
+    if ( ( color & Color::ALL ) == 0 ) {
+        DEBUG_LOG( DBG_GAME, DBG_WARN, "Unknown player: " << Color::String( color ) << "(" << static_cast<int>( color ) << ")" )
 
-        resource = _getKingdomStartingResources( difficultyLevel );
+        return;
+    }
 
-        // Some human players can have handicap for resources.
-        const Player * player = Players::Get( color );
-        assert( player != nullptr );
-        resource = getHandicapDependentIncome( resource, player->getHandicapStatus() );
-    }
-    else {
-        DEBUG_LOG( DBG_GAME, DBG_WARN, "Kingdom: unknown player: " << Color::String( color ) << "(" << static_cast<int>( color ) << ")" )
-    }
+    resource = _getKingdomStartingResources( Game::getDifficulty() );
+
+    const Player * player = Players::Get( color );
+    assert( player != nullptr );
+
+    // Some human players can have handicap for resources.
+    resource = getHandicapDependentIncome( resource, player->getHandicapStatus() );
 }
 
 void Kingdom::clear()
@@ -244,19 +240,6 @@ void Kingdom::ActionNewDayResourceUpdate( const std::function<void( const EventD
     // Skip the income for the first day
     if ( world.CountDay() > 1 ) {
         AddFundsResource( GetIncome() );
-
-        // Resource bonuses from campaign awards
-        if ( isControlHuman() && Settings::Get().isCampaignGameType() ) {
-            const std::vector<Campaign::CampaignAwardData> campaignAwards = Campaign::CampaignSaveData::Get().getObtainedCampaignAwards();
-
-            for ( size_t i = 0; i < campaignAwards.size(); ++i ) {
-                if ( campaignAwards[i]._type != Campaign::CampaignAwardData::TYPE_RESOURCE_BONUS ) {
-                    continue;
-                }
-
-                AddFundsResource( Funds( campaignAwards[i]._subType, campaignAwards[i]._amount ) );
-            }
-        }
     }
 
     const bool isAIPlayer = ( GetControl() == CONTROL_AI );
@@ -321,32 +304,33 @@ void Kingdom::AddHero( Heroes * hero )
 
 void Kingdom::RemoveHero( const Heroes * hero )
 {
-    if ( hero != nullptr ) {
-        if ( !heroes.empty() ) {
-            auto it = std::find( heroes.begin(), heroes.end(), hero );
-            assert( it != heroes.end() );
-            if ( it != heroes.end() ) {
-                heroes.erase( it );
-            }
-        }
-
-        Player * player = Players::Get( GetColor() );
-
-        if ( player && player->GetFocus().GetHeroes() == hero ) {
-            player->GetFocus().Reset();
-        }
-
-        assert( hero != nullptr );
-
-        AI::Get().HeroesRemove( *hero );
-    }
-    else {
-        // Why are trying to delete a non existing hero?
+    if ( hero == nullptr ) {
+        // Why are you trying to delete a non-existing hero?
         assert( 0 );
+        return;
     }
 
-    if ( isLoss() )
+    if ( !heroes.empty() ) {
+        auto it = std::find( heroes.begin(), heroes.end(), hero );
+        assert( it != heroes.end() );
+        if ( it != heroes.end() ) {
+            heroes.erase( it );
+        }
+    }
+
+    Player * player = Players::Get( GetColor() );
+
+    if ( player && player->GetFocus().GetHeroes() == hero ) {
+        player->GetFocus().Reset();
+    }
+
+    assert( hero != nullptr );
+
+    AI::Get().HeroesRemove( *hero );
+
+    if ( isLoss() ) {
         LossPostActions();
+    }
 }
 
 void Kingdom::AddCastle( const Castle * castle )
@@ -469,7 +453,7 @@ void Kingdom::SetVisited( int32_t index, const MP2::MapObjectType objectType )
 
 bool Kingdom::isValidKingdomObject( const Maps::Tiles & tile, const MP2::MapObjectType objectType ) const
 {
-    if ( !MP2::isActionObject( objectType ) )
+    if ( !MP2::isInGameActionObject( objectType ) )
         return false;
 
     if ( isVisited( tile.GetIndex(), objectType ) )
@@ -709,7 +693,7 @@ Funds Kingdom::GetIncome( int type /* INCOME_ALL */ ) const
         }
     }
 
-    if ( ( type & INCOME_CAMPAIGN_BONUS ) && Settings::Get().isCampaignGameType() ) {
+    if ( isControlHuman() && ( type & INCOME_CAMPAIGN_BONUS ) && Settings::Get().isCampaignGameType() ) {
         const std::vector<Campaign::CampaignAwardData> awards = Campaign::CampaignSaveData::Get().getObtainedCampaignAwards();
         for ( const Campaign::CampaignAwardData & award : awards ) {
             if ( award._type != Campaign::CampaignAwardData::TYPE_RESOURCE_BONUS ) {
@@ -839,16 +823,6 @@ Kingdom & Kingdoms::GetKingdom( int color )
     return kingdoms[6];
 }
 
-void Kingdom::SetLastBattleWinHero( const Heroes & hero )
-{
-    _lastBattleWinHeroID = hero.GetID();
-}
-
-Heroes * Kingdom::GetLastBattleWinHero() const
-{
-    return Heroes::UNKNOWN != _lastBattleWinHeroID ? world.GetHeroes( _lastBattleWinHeroID ) : nullptr;
-}
-
 void Kingdom::appendSurrenderedHero( Heroes & hero )
 {
     recruits.appendSurrenderedHero( hero, world.CountDay() );
@@ -897,6 +871,7 @@ void Kingdoms::AddHeroes( const AllHeroes & heroes )
     for ( Heroes * hero : heroes ) {
         assert( hero != nullptr );
 
+        // Skip neutral heroes.
         if ( hero->GetColor() != Color::NONE ) {
             GetKingdom( hero->GetColor() ).AddHero( hero );
         }
@@ -906,9 +881,12 @@ void Kingdoms::AddHeroes( const AllHeroes & heroes )
 void Kingdoms::AddCastles( const AllCastles & castles )
 {
     for ( const Castle * castle : castles ) {
-        // skip gray color
-        if ( castle->GetColor() )
+        assert( castle != nullptr );
+
+        // Skip neutral castles and towns.
+        if ( castle->GetColor() != Color::NONE ) {
             GetKingdom( castle->GetColor() ).AddCastle( castle );
+        }
     }
 }
 
@@ -1006,18 +984,19 @@ cost_t Kingdom::_getKingdomStartingResources( const int difficulty ) const
 StreamBase & operator<<( StreamBase & msg, const Kingdom & kingdom )
 {
     return msg << kingdom.modes << kingdom.color << kingdom.resource << kingdom.lost_town_days << kingdom.castles << kingdom.heroes << kingdom.recruits
-               << kingdom.visit_object << kingdom.puzzle_maps << kingdom.visited_tents_colors << kingdom._lastBattleWinHeroID << kingdom._topCastleInKingdomView
-               << kingdom._topHeroInKingdomView;
+               << kingdom.visit_object << kingdom.puzzle_maps << kingdom.visited_tents_colors << kingdom._topCastleInKingdomView << kingdom._topHeroInKingdomView;
 }
 
 StreamBase & operator>>( StreamBase & msg, Kingdom & kingdom )
 {
     msg >> kingdom.modes >> kingdom.color >> kingdom.resource >> kingdom.lost_town_days >> kingdom.castles >> kingdom.heroes >> kingdom.recruits >> kingdom.visit_object
-        >> kingdom.puzzle_maps >> kingdom.visited_tents_colors >> kingdom._lastBattleWinHeroID;
+        >> kingdom.puzzle_maps >> kingdom.visited_tents_colors;
 
-    static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_1010_RELEASE, "Remove the logic below." );
-    if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_1010_RELEASE ) {
-        ++kingdom._lastBattleWinHeroID;
+    static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_PRE2_1100_RELEASE, "Remove the logic below." );
+    if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_PRE2_1100_RELEASE ) {
+        int dummy;
+
+        msg >> dummy;
     }
 
     return msg >> kingdom._topCastleInKingdomView >> kingdom._topHeroInKingdomView;
