@@ -1,6 +1,6 @@
 /***************************************************************************
  *   fheroes2: https://github.com/ihhub/fheroes2                           *
- *   Copyright (C) 2019 - 2023                                             *
+ *   Copyright (C) 2019 - 2024                                             *
  *                                                                         *
  *   Free Heroes2 Engine: http://sourceforge.net/projects/fheroes2         *
  *   Copyright (C) 2009 by Andrey Afletdinov <fheroes2@gmail.com>          *
@@ -26,6 +26,7 @@
 #include <algorithm>
 #include <cassert>
 #include <cstring>
+#include <functional>
 #include <limits>
 #include <list>
 #include <locale>
@@ -85,7 +86,7 @@ namespace
     MapsFileInfoList getValidMaps( const ListFiles & mapFiles, const bool isMultiplayer, const bool isForEditor, const bool isOriginalMapFormat )
     {
         // create a list of unique maps (based on the map file name) and filter it by the preferred number of players
-        std::map<std::string, Maps::FileInfo> uniqueMaps;
+        std::map<std::string, Maps::FileInfo, std::less<>> uniqueMaps;
 
         const Settings & conf = Settings::Get();
         const int prefNumOfPlayers = conf.PreferablyCountPlayers();
@@ -392,11 +393,28 @@ bool Maps::FileInfo::readResurrectionMap( std::string filePath, const bool isFor
 {
     Reset();
 
-    Maps::Map_Format::MapFormat map;
+    Maps::Map_Format::BaseMapFormat map;
     if ( !Maps::Map_Format::loadBaseMap( filePath, map ) ) {
         return false;
     }
 
+    if ( !loadResurrectionMap( map, std::move( filePath ) ) ) {
+        return false;
+    }
+
+    if ( !isForEditor && colorsAvailableForHumans == 0 ) {
+        // This is not a valid map since no human players exist so it cannot be played.
+        DEBUG_LOG( DBG_GAME, DBG_WARN, "Map " << filename << " does not contain any human players." )
+        return false;
+    }
+
+    return true;
+}
+
+bool Maps::FileInfo::loadResurrectionMap( const Map_Format::BaseMapFormat & map, std::string filePath )
+{
+    Reset();
+
     filename = std::move( filePath );
 
     difficulty = map.difficulty;
@@ -404,21 +422,99 @@ bool Maps::FileInfo::readResurrectionMap( std::string filePath, const bool isFor
     width = static_cast<uint16_t>( map.size );
     height = static_cast<uint16_t>( map.size );
 
-    name = std::move( map.name );
-    description = std::move( map.description );
+    name = map.name;
+    description = map.description;
+
+    assert( ( map.availablePlayerColors & map.humanPlayerColors ) == map.humanPlayerColors );
+    assert( ( map.availablePlayerColors & map.computerPlayerColors ) == map.computerPlayerColors );
+    assert( ( map.availablePlayerColors & ( map.humanPlayerColors | map.computerPlayerColors ) ) == map.availablePlayerColors );
 
     kingdomColors = map.availablePlayerColors;
     colorsAvailableForHumans = map.humanPlayerColors;
     colorsAvailableForComp = map.computerPlayerColors;
 
-    version = GameVersion::RESURRECTION;
+    races = map.playerRace;
 
-    if ( !isForEditor && colorsAvailableForHumans == 0 ) {
-        // This is not a valid map since no human players exist so it cannot be played.
-        DEBUG_LOG( DBG_GAME, DBG_WARN, "Map " << filename << " does not contain any human players." )
-        return false;
+    victoryConditionType = map.victoryConditionType;
+    compAlsoWins = map.isVictoryConditionApplicableForAI;
+    allowNormalVictory = map.allowNormalVictory;
+
+    lossConditionType = map.lossConditionType;
+
+    switch ( lossConditionType ) {
+    case LOSS_EVERYTHING:
+        // No special conditions of loss.
+        // So, no metadata should exist.
+        assert( map.lossConditionMetadata.empty() );
+        break;
+    case LOSS_TOWN:
+    case LOSS_HERO:
+        // Town or hero loss metadata must include the following:
+        // - X position of the object
+        // - Y position of the object
+        // - color of the object (the color is needed to modify the game for multi-player)
+        assert( map.victoryConditionMetadata.size() == 3 );
+        lossConditionParams[0] = static_cast<uint16_t>( map.victoryConditionMetadata[0] );
+        lossConditionParams[1] = static_cast<uint16_t>( map.victoryConditionMetadata[1] );
+        assert( ( map.victoryConditionMetadata[2] & map.humanPlayerColors ) == map.victoryConditionMetadata[2] );
+        break;
+    case LOSS_OUT_OF_TIME:
+        assert( map.victoryConditionMetadata.size() == 1 );
+        lossConditionParams[0] = static_cast<uint16_t>( map.victoryConditionMetadata[0] );
+        break;
+    default:
+        // Did you add a new loss condition? Add the logic above!
+        assert( 0 );
+        break;
     }
 
+    switch ( victoryConditionType ) {
+    case VICTORY_DEFEAT_EVERYONE:
+        // Since this is a normal victory condition.
+        // So, no metadata should exist.
+        assert( map.victoryConditionMetadata.empty() );
+        compAlsoWins = true;
+        allowNormalVictory = true;
+        break;
+    case VICTORY_CAPTURE_TOWN:
+    case VICTORY_KILL_HERO:
+        // Town or hero capture metadata must include the following:
+        // - X position of the object
+        // - Y position of the object
+        // - color of the object (the color is needed to modify the game for multi-player)
+        assert( map.victoryConditionMetadata.size() == 3 );
+        victoryConditionParams[0] = static_cast<uint16_t>( map.victoryConditionMetadata[0] );
+        victoryConditionParams[1] = static_cast<uint16_t>( map.victoryConditionMetadata[1] );
+        assert( ( map.victoryConditionMetadata[2] & map.availablePlayerColors ) == map.victoryConditionMetadata[2] );
+        break;
+    case VICTORY_OBTAIN_ARTIFACT:
+    case VICTORY_COLLECT_ENOUGH_GOLD:
+        assert( map.victoryConditionMetadata.size() == 1 );
+        victoryConditionParams[0] = static_cast<uint16_t>( map.victoryConditionMetadata[0] );
+        break;
+    case VICTORY_DEFEAT_OTHER_SIDE:
+        // As of now only 2 alliances are supported.
+        assert( map.alliances.size() == 2 );
+
+        assert( std::all_of( map.alliances.begin(), map.alliances.end(),
+                             [&map = std::as_const( map )]( const uint8_t color ) { return ( color & map.availablePlayerColors ) == color; } ) );
+
+        FillUnions( map.alliances[0], map.alliances[1] );
+        break;
+    default:
+        // Did you add a new victory condition? Add the logic for it!
+        assert( 0 );
+        break;
+    }
+
+    for ( size_t i = 0; i < races.size(); ++i ) {
+        if ( races[i] == Race::RAND ) {
+            colorsOfRandomRaces |= static_cast<uint8_t>( 1 << i );
+        }
+    }
+
+    version = GameVersion::RESURRECTION;
+
     return true;
 }
 
@@ -618,7 +714,12 @@ MapsFileInfoList Maps::getAllMapFileInfos( const bool isForEditor, const bool is
         }
     }
 
-    std::sort( validMaps.begin(), validMaps.end(), Maps::FileInfo::sortByMapName );
+    if ( isForEditor ) {
+        std::sort( validMaps.begin(), validMaps.end(), Maps::FileInfo::sortByFileName );
+    }
+    else {
+        std::sort( validMaps.begin(), validMaps.end(), Maps::FileInfo::sortByMapName );
+    }
 
     return validMaps;
 }
@@ -633,7 +734,12 @@ MapsFileInfoList Maps::getResurrectionMapFileInfos( const bool isForEditor, cons
     const ListFiles maps = Settings::FindFiles( "maps", ".fh2m", false );
     MapsFileInfoList validMaps = getValidMaps( maps, isMultiplayer, isForEditor, false );
 
-    std::sort( validMaps.begin(), validMaps.end(), Maps::FileInfo::sortByMapName );
+    if ( isForEditor ) {
+        std::sort( validMaps.begin(), validMaps.end(), Maps::FileInfo::sortByFileName );
+    }
+    else {
+        std::sort( validMaps.begin(), validMaps.end(), Maps::FileInfo::sortByMapName );
+    }
 
     return validMaps;
 }
