@@ -79,7 +79,7 @@ public:
     bool Save( const std::string_view fileName ) const;
 
     std::string String() const;
-    void SetCurrentFileInfo( const Maps::FileInfo & );
+    void SetCurrentFileInfo( Maps::FileInfo fi );
 
     const Maps::FileInfo & getCurrentMapInfo() const
     {
@@ -187,7 +187,9 @@ public:
     bool isBattleShowDamageInfoEnabled() const;
     bool isHideInterfaceEnabled() const;
     bool isEvilInterfaceEnabled() const;
-    bool isEditorEnabled() const;
+
+    bool isEditorAnimationEnabled() const;
+    bool isEditorPassabilityEnabled() const;
 
     bool LoadedGameVersion() const
     {
@@ -260,6 +262,9 @@ public:
 
     bool setGameLanguage( const std::string & language );
 
+    void setEditorAnimation( const bool enable );
+    void setEditorPassability( const bool enable );
+
     int SoundVolume() const
     {
         return sound_volume;
@@ -353,8 +358,11 @@ private:
 
     static void setDebug( int debug );
 
-    // Global game options (GLOBAL_)
-    BitModes _optGlobal;
+    // Game related options.
+    BitModes _gameOptions;
+
+    // Editor related options.
+    BitModes _editorOptions;
 
     fheroes2::ResolutionInfo _resolutionInfo;
     int _gameDifficulty;
