@@ -26,6 +26,9 @@
 #include <cassert>
 #include <cstddef>
 #include <cstdint>
+#include <functional>
+#include <list>
+#include <map>
 #include <memory>
 #include <ostream>
 #include <set>
@@ -45,6 +48,7 @@
 #include "logging.h"
 #include "map_format_helper.h"
 #include "map_format_info.h"
+#include "map_object_info.h"
 #include "maps.h"
 #include "maps_fileinfo.h"
 #include "maps_objects.h"
@@ -56,6 +60,7 @@
 #include "players.h"
 #include "race.h"
 #include "rand.h"
+#include "resource.h"
 #include "serialize.h"
 #include "settings.h"
 #include "world.h"
@@ -84,7 +89,7 @@ namespace
     {
         // Find castles with no names.
         std::vector<Castle *> castleWithNoName;
-        std::set<std::string> castleNames;
+        std::set<std::string, std::less<>> castleNames;
 
         for ( Castle * castle : castles ) {
             if ( castle == nullptr ) {
@@ -597,6 +602,9 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
                 if ( MP2::MP2_RIDDLE_STRUCTURE_MIN_SIZE <= pblock.size() && 0x00 == pblock[0] ) {
                     MapSphinx * obj = new MapSphinx();
                     obj->LoadFromMP2( objectTileId, pblock );
+
+                    obj->validate();
+
                     map_objects.add( obj );
                 }
                 break;
@@ -682,6 +690,405 @@ bool World::loadResurrectionMap( const std::string & filename )
         return false;
     }
 
+    if ( !Maps::updateMapPlayers( map ) ) {
+        return false;
+    }
+
+    // Read and populate objects.
+    const auto & townObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::KINGDOM_TOWNS );
+    const auto & heroObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::KINGDOM_HEROES );
+    const auto & miscellaneousObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_MISCELLANEOUS );
+    const auto & waterObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_WATER );
+    const auto & artifactObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_ARTIFACTS );
+
+#if defined( WITH_DEBUG )
+    std::set<uint32_t> standardMetadataUIDs;
+    std::set<uint32_t> castleMetadataUIDs;
+    std::set<uint32_t> heroMetadataUIDs;
+    std::set<uint32_t> sphinxMetadataUIDs;
+    std::set<uint32_t> signMetadataUIDs;
+    std::set<uint32_t> adventureMapEventMetadataUIDs;
+#endif
+
+    std::set<fheroes2::Point> hiredHeroPos;
+
+    for ( size_t tileId = 0; tileId < map.tiles.size(); ++tileId ) {
+        const auto & tile = map.tiles[tileId];
+
+        for ( const auto & object : tile.objects ) {
+            if ( object.group == Maps::ObjectGroup::KINGDOM_TOWNS ) {
+#if defined( WITH_DEBUG )
+                castleMetadataUIDs.emplace( object.id );
+#endif
+                assert( map.castleMetadata.find( object.id ) != map.castleMetadata.end() );
+                auto & castleInfo = map.castleMetadata[object.id];
+
+                const int color = Color::IndexToColor( Maps::getTownColorIndex( map, tileId, object.id ) );
+
+                int race = Race::IndexToRace( static_cast<int>( townObjects[object.index].metadata[0] ) );
+                const bool isRandom = ( race == Race::RAND );
+
+                if ( isRandom ) {
+                    assert( townObjects[object.index].objectType == MP2::OBJ_RANDOM_CASTLE || townObjects[object.index].objectType == MP2::OBJ_RANDOM_TOWN );
+
+                    if ( ( color & Color::ALL ) == 0 ) {
+                        // This is a neutral town.
+                        race = Race::Rand();
+                    }
+                    else {
+                        const Kingdom & kingdom = GetKingdom( color );
+                        assert( kingdom.GetColor() == color );
+
+                        race = static_cast<uint8_t>( kingdom.GetRace() );
+                    }
+                }
+
+                assert( ( std::find( castleInfo.builtBuildings.begin(), castleInfo.builtBuildings.end(), BUILD_CASTLE ) != castleInfo.builtBuildings.end() )
+                        == ( townObjects[object.index].metadata[1] != 0 ) );
+                assert( ( std::find( castleInfo.builtBuildings.begin(), castleInfo.builtBuildings.end(), BUILD_TENT ) != castleInfo.builtBuildings.end() )
+                        == ( townObjects[object.index].metadata[1] == 0 ) );
+
+                Castle * castle = new Castle( static_cast<int32_t>( tileId ) % width, static_cast<int32_t>( tileId ) / width, race );
+                castle->SetColor( color );
+                castle->loadFromResurrectionMap( castleInfo );
+
+                vec_castles.AddCastle( castle );
+
+                if ( isRandom ) {
+                    Maps::UpdateCastleSprite( castle->GetCenter(), race, castle->isCastle(), true );
+                    Maps::ReplaceRandomCastleObjectId( castle->GetCenter() );
+                }
+
+                map_captureobj.Set( static_cast<int32_t>( tileId ), MP2::OBJ_CASTLE, color );
+            }
+            else if ( object.group == Maps::ObjectGroup::KINGDOM_HEROES ) {
+#if defined( WITH_DEBUG )
+                heroMetadataUIDs.emplace( object.id );
+#endif
+
+                assert( map.heroMetadata.find( object.id ) != map.heroMetadata.end() );
+
+                const auto & metadata = heroObjects[object.index].metadata;
+                auto & heroInfo = map.heroMetadata[object.id];
+
+                const int color = Color::IndexToColor( static_cast<int>( metadata[0] ) );
+
+                // Check the race correctness.
+                assert( heroInfo.race == Race::IndexToRace( metadata[1] ) );
+
+                // Heroes can not be neutral.
+                assert( color != Color::NONE );
+
+                const Kingdom & kingdom = GetKingdom( color );
+
+                // Set race for random hero according to the kingdom's race.
+                if ( heroInfo.race == Race::RAND ) {
+                    heroInfo.race = static_cast<uint8_t>( kingdom.GetRace() );
+                }
+
+                // Check if the kingdom has exceeded the limit on hired heroes
+                if ( kingdom.AllowRecruitHero( false ) ) {
+                    Heroes * hero = GetHeroForHire( heroInfo.race );
+                    if ( hero != nullptr ) {
+                        hero->SetCenter( { static_cast<int32_t>( tileId ) % width, static_cast<int32_t>( tileId ) / width } );
+
+                        hero->SetColor( color );
+
+                        hero->applyHeroMetadata( heroInfo, false, false );
+
+                        hiredHeroPos.emplace( static_cast<int32_t>( tileId ) % width, static_cast<int32_t>( tileId ) / width );
+                    }
+                    else {
+                        VERBOSE_LOG( "A hero at position " << tileId << " with UID " << object.id << " failed to be hired." )
+                    }
+                }
+                else {
+                    VERBOSE_LOG( "A hero at position " << tileId << " with UID " << object.id << " cannot be hired." )
+                }
+            }
+            else if ( object.group == Maps::ObjectGroup::MONSTERS ) {
+#if defined( WITH_DEBUG )
+                standardMetadataUIDs.emplace( object.id );
+#endif
+                assert( map.standardMetadata.find( object.id ) != map.standardMetadata.end() );
+                auto & objectInfo = map.standardMetadata[object.id];
+
+                std::array<uint32_t, 3> & tileData = vec_tiles[static_cast<int32_t>( tileId )].metadata();
+
+                for ( size_t idx = 0; idx < objectInfo.metadata.size(); ++idx ) {
+                    tileData[idx] = static_cast<uint32_t>( objectInfo.metadata[idx] );
+                }
+            }
+            else if ( object.group == Maps::ObjectGroup::ADVENTURE_MISCELLANEOUS ) {
+                assert( object.index < miscellaneousObjects.size() );
+
+                const auto objectType = miscellaneousObjects[object.index].objectType;
+                switch ( objectType ) {
+                case MP2::OBJ_EVENT: {
+#if defined( WITH_DEBUG )
+                    adventureMapEventMetadataUIDs.emplace( object.id );
+#endif
+                    assert( map.adventureMapEventMetadata.find( object.id ) != map.adventureMapEventMetadata.end() );
+                    auto & eventInfo = map.adventureMapEventMetadata[object.id];
+
+                    eventInfo.humanPlayerColors = eventInfo.humanPlayerColors & map.humanPlayerColors;
+                    eventInfo.computerPlayerColors = eventInfo.computerPlayerColors & map.computerPlayerColors;
+
+                    // TODO: change MapEvent to support map format functionality.
+                    MapEvent * eventObject = new MapEvent();
+                    eventObject->resources = eventInfo.resources;
+                    eventObject->artifact = eventInfo.artifact;
+                    if ( eventInfo.artifact == Artifact::SPELL_SCROLL ) {
+                        eventObject->artifact.SetSpell( eventInfo.artifactMetadata );
+                    }
+
+                    const int humanColors = Players::HumanColors() & eventInfo.humanPlayerColors;
+                    const int computerColors = ( ~Players::HumanColors() ) & eventInfo.computerPlayerColors;
+
+                    eventObject->computer = ( computerColors != 0 );
+                    eventObject->colors = humanColors | computerColors;
+                    eventObject->message = std::move( eventInfo.message );
+                    eventObject->isSingleTimeEvent = !eventInfo.isRecurringEvent;
+
+                    eventObject->setUIDAndIndex( static_cast<int32_t>( tileId ) );
+                    map_objects.add( eventObject );
+
+                    break;
+                }
+                case MP2::OBJ_JAIL: {
+#if defined( WITH_DEBUG )
+                    heroMetadataUIDs.emplace( object.id );
+#endif
+
+                    assert( map.heroMetadata.find( object.id ) != map.heroMetadata.end() );
+
+                    auto & heroInfo = map.heroMetadata[object.id];
+
+                    const int color = Color::NONE;
+
+                    if ( heroInfo.race == Race::RAND ) {
+                        heroInfo.race = static_cast<uint8_t>( Race::Rand() );
+                    }
+
+                    Heroes * hero = GetHeroForHire( heroInfo.race );
+                    if ( hero != nullptr ) {
+                        hero->SetCenter( { static_cast<int32_t>( tileId ) % width, static_cast<int32_t>( tileId ) / width } );
+
+                        hero->SetColor( color );
+
+                        hero->applyHeroMetadata( heroInfo, true, false );
+                    }
+                    else {
+                        VERBOSE_LOG( "A hero at position " << tileId << " with UID " << object.id << " failed to be hired." )
+                    }
+                    break;
+                }
+                case MP2::OBJ_SIGN: {
+#if defined( WITH_DEBUG )
+                    signMetadataUIDs.emplace( object.id );
+#endif
+                    assert( map.signMetadata.find( object.id ) != map.signMetadata.end() );
+                    auto & signInfo = map.signMetadata[object.id];
+
+                    MapSign * signObject = new MapSign();
+                    signObject->message = std::move( signInfo.message );
+                    signObject->setUIDAndIndex( static_cast<int32_t>( tileId ) );
+                    if ( signObject->message.empty() ) {
+                        signObject->setDefaultMessage();
+                    }
+
+                    map_objects.add( signObject );
+
+                    break;
+                }
+                case MP2::OBJ_SPHINX: {
+#if defined( WITH_DEBUG )
+                    sphinxMetadataUIDs.emplace( object.id );
+#endif
+
+                    assert( map.sphinxMetadata.find( object.id ) != map.sphinxMetadata.end() );
+                    auto & sphinxInfo = map.sphinxMetadata[object.id];
+
+                    MapSphinx * sphinxObject = new MapSphinx();
+
+                    sphinxObject->riddle = std::move( sphinxInfo.riddle );
+
+                    for ( auto & answer : sphinxInfo.answers ) {
+                        if ( !answer.empty() ) {
+                            sphinxObject->answers.push_back( std::move( answer ) );
+                        }
+                        else {
+                            // How is it even possible?
+                            assert( 0 );
+                        }
+                    }
+
+                    sphinxObject->resources = sphinxInfo.resources;
+                    sphinxObject->artifact = sphinxInfo.artifact;
+                    if ( sphinxInfo.artifact == Artifact::SPELL_SCROLL ) {
+                        sphinxObject->artifact.SetSpell( sphinxInfo.artifactMetadata );
+                    }
+
+                    sphinxObject->validate();
+
+                    sphinxObject->setUIDAndIndex( static_cast<int32_t>( tileId ) );
+
+                    // The original game assumes answers only to be up to 4 characters.
+                    // However, it seems logically incorrect.
+                    sphinxObject->isTruncatedAnswer = false;
+
+                    map_objects.add( sphinxObject );
+
+                    break;
+                }
+                default:
+                    // Other objects do not have metadata as of now.
+                    break;
+                }
+            }
+            else if ( object.group == Maps::ObjectGroup::ADVENTURE_WATER ) {
+                assert( object.index < waterObjects.size() );
+
+                if ( waterObjects[object.index].objectType == MP2::OBJ_BOTTLE ) {
+#if defined( WITH_DEBUG )
+                    signMetadataUIDs.emplace( object.id );
+#endif
+                    assert( map.signMetadata.find( object.id ) != map.signMetadata.end() );
+                    auto & signInfo = map.signMetadata[object.id];
+
+                    MapSign * signObject = new MapSign();
+                    signObject->message = std::move( signInfo.message );
+                    signObject->setUIDAndIndex( static_cast<int32_t>( tileId ) );
+                    if ( signObject->message.empty() ) {
+                        signObject->setDefaultMessage();
+                    }
+
+                    map_objects.add( signObject );
+                }
+            }
+            else if ( object.group == Maps::ObjectGroup::ADVENTURE_ARTIFACTS ) {
+#if defined( WITH_DEBUG )
+                standardMetadataUIDs.emplace( object.id );
+#endif
+
+                assert( map.standardMetadata.find( object.id ) != map.standardMetadata.end() );
+                auto & objectInfo = map.standardMetadata[object.id];
+
+                std::array<uint32_t, 3> & tileData = vec_tiles[static_cast<int32_t>( tileId )].metadata();
+
+                for ( size_t idx = 0; idx < objectInfo.metadata.size(); ++idx ) {
+                    tileData[idx] = static_cast<uint32_t>( objectInfo.metadata[idx] );
+                }
+
+                assert( object.index < artifactObjects.size() );
+                if ( artifactObjects[object.index].objectType == MP2::OBJ_ARTIFACT && artifactObjects[object.index].metadata[0] == Artifact::SPELL_SCROLL ) {
+                    // This is a hack we need to do since in the original game spells start from 0.
+                    // TODO: fix this hack.
+                    assert( tileData[0] > 0 );
+                    tileData[0] = tileData[0] - 1U;
+                }
+            }
+        }
+    }
+
+#if defined( WITH_DEBUG )
+    assert( standardMetadataUIDs.size() == map.standardMetadata.size() );
+    assert( castleMetadataUIDs.size() == map.castleMetadata.size() );
+    assert( heroMetadataUIDs.size() == map.heroMetadata.size() );
+    assert( sphinxMetadataUIDs.size() == map.sphinxMetadata.size() );
+    assert( signMetadataUIDs.size() == map.signMetadata.size() );
+    assert( adventureMapEventMetadataUIDs.size() == map.adventureMapEventMetadata.size() );
+
+    for ( const uint32_t uid : standardMetadataUIDs ) {
+        assert( map.standardMetadata.find( uid ) != map.standardMetadata.end() );
+    }
+
+    for ( const uint32_t uid : castleMetadataUIDs ) {
+        assert( map.castleMetadata.find( uid ) != map.castleMetadata.end() );
+    }
+
+    for ( const uint32_t uid : heroMetadataUIDs ) {
+        assert( map.heroMetadata.find( uid ) != map.heroMetadata.end() );
+    }
+
+    for ( const uint32_t uid : sphinxMetadataUIDs ) {
+        assert( map.sphinxMetadata.find( uid ) != map.sphinxMetadata.end() );
+    }
+
+    for ( const uint32_t uid : signMetadataUIDs ) {
+        assert( map.signMetadata.find( uid ) != map.signMetadata.end() );
+    }
+
+    for ( const uint32_t uid : adventureMapEventMetadataUIDs ) {
+        assert( map.adventureMapEventMetadata.find( uid ) != map.adventureMapEventMetadata.end() );
+    }
+#endif
+
+    // Load daily events.
+    for ( auto & event : map.dailyEvents ) {
+        event.humanPlayerColors = event.humanPlayerColors & map.humanPlayerColors;
+        event.computerPlayerColors = event.computerPlayerColors & map.computerPlayerColors;
+
+        // TODO: modify EventDate structure to have more flexibility.
+        auto & newEvent = vec_eventsday.emplace_back();
+
+        const int humanColors = Players::HumanColors() & event.humanPlayerColors;
+        const int computerColors = ( ~Players::HumanColors() ) & event.computerPlayerColors;
+
+        newEvent.message = std::move( event.message );
+        newEvent.colors = ( humanColors | computerColors );
+        newEvent.isApplicableForAIPlayers = ( computerColors != 0 );
+
+        newEvent.firstOccurrenceDay = event.firstOccurrenceDay;
+        newEvent.repeatPeriodInDays = event.repeatPeriodInDays;
+        newEvent.resource = event.resources;
+    }
+
+    // Load rumors.
+    for ( auto & rumor : map.rumors ) {
+        _rumors.emplace_back( std::move( rumor ) );
+    }
+
+    // Verify that a capture or loss object exists.
+    if ( map.lossConditionType == Maps::FileInfo::LOSS_HERO ) {
+        auto iter = hiredHeroPos.find( { static_cast<int32_t>( map.lossConditionMetadata[0] ), static_cast<int32_t>( map.lossConditionMetadata[1] ) } );
+        if ( iter == hiredHeroPos.end() ) {
+            VERBOSE_LOG( "A hero at position [" << map.lossConditionMetadata[0] << ", " << map.lossConditionMetadata[1] << "] does not exist." )
+            return false;
+        }
+    }
+    else if ( map.lossConditionType == Maps::FileInfo::LOSS_TOWN ) {
+        const Castle * castle = vec_castles.Get( { static_cast<int32_t>( map.lossConditionMetadata[0] ), static_cast<int32_t>( map.lossConditionMetadata[1] ) } );
+        if ( castle == nullptr ) {
+            VERBOSE_LOG( "A castle at position [" << map.lossConditionMetadata[0] << ", " << map.lossConditionMetadata[1] << "] does not exist." )
+            return false;
+        }
+    }
+
+    if ( map.victoryConditionType == Maps::FileInfo::VICTORY_KILL_HERO ) {
+        auto iter = hiredHeroPos.find( { static_cast<int32_t>( map.victoryConditionMetadata[0] ), static_cast<int32_t>( map.victoryConditionMetadata[1] ) } );
+        if ( iter == hiredHeroPos.end() ) {
+            VERBOSE_LOG( "A hero at position [" << map.victoryConditionMetadata[0] << ", " << map.victoryConditionMetadata[1] << "] does not exist." )
+            return false;
+        }
+    }
+    else if ( map.victoryConditionType == Maps::FileInfo::VICTORY_CAPTURE_TOWN ) {
+        const Castle * castle = vec_castles.Get( { static_cast<int32_t>( map.victoryConditionMetadata[0] ), static_cast<int32_t>( map.victoryConditionMetadata[1] ) } );
+        if ( castle == nullptr ) {
+            VERBOSE_LOG( "A castle at position [" << map.victoryConditionMetadata[0] << ", " << map.victoryConditionMetadata[1] << "] does not exist." )
+            return false;
+        }
+    }
+
+    fixCastleNames( vec_castles );
+
+    const Maps::FileInfo & mapInfo = Settings::Get().getCurrentMapInfo();
+
+    // do not let the player get a random artifact that allows him to win the game
+    if ( ( mapInfo.ConditionWins() & GameOver::WINS_ARTIFACT ) == GameOver::WINS_ARTIFACT && !mapInfo.WinsFindUltimateArtifact() ) {
+        fheroes2::ExcludeArtifactFromRandom( mapInfo.WinsFindArtifactID() );
+    }
+
     // Clear artifact flags to correctly generate random artifacts.
     fheroes2::ResetArtifactStats();
 
@@ -932,8 +1339,9 @@ bool World::updateTileMetadata( Maps::Tiles & tile, const MP2::MapObjectType obj
 
     case MP2::OBJ_HERO: {
         // remove map editor sprite
-        if ( tile.getObjectIcnType() == MP2::OBJ_ICN_TYPE_MINIHERO )
+        if ( tile.getObjectIcnType() == MP2::OBJ_ICN_TYPE_MINIHERO ) {
             tile.Remove( tile.GetObjectUID() );
+        }
 
         Heroes * chosenHero = GetHeroes( Maps::GetPoint( tile.GetIndex() ) );
         assert( chosenHero != nullptr );
